package CDC2023
    extends Modelica.Icons.Package;
  package Vehicles
      extends VehicleDynamics.Utilities.Icons.VehiclesPackage;
    package Interfaces
        extends Modelica.Icons.Package;
      connector VehicleOutputs "Vehicle Outputs"
          .Modelica.Blocks.Interfaces.RealOutput phi_p(final quantity="Angle", final unit="rad")
          "Body roll angle";
          .Modelica.Blocks.Interfaces.RealOutput phi_q(final quantity="Angle", final unit="rad")
          "Body pitch angle";
          .Modelica.Blocks.Interfaces.RealOutput phi_r(final quantity="Angle", final unit="rad")
          "Body yaw angle";
          .Modelica.Blocks.Interfaces.RealOutput omg_p(final quantity="AngularVelocity", final unit="rad/s")
          "Body roll speed";
          .Modelica.Blocks.Interfaces.RealOutput omg_q(final quantity="AngularVelocity", final unit="rad/s")
          "Body pitch speed";
          .Modelica.Blocks.Interfaces.RealOutput omg_r(final quantity="AngularVelocity", final unit="rad/s")
          "Body yaw speed";
          .Modelica.Blocks.Interfaces.RealOutput a_z(final quantity="Acceleration", final unit="m/s2")
          "Body vertical acceleration";
          .Modelica.Blocks.Interfaces.RealOutput a_y(final quantity="Acceleration", final unit="m/s2")
          "Body lateral acceleration";
          .Modelica.Blocks.Interfaces.RealOutput a_x(final quantity="Acceleration", final unit="m/s2")
          "Body longitudinal acceleration";
          .Modelica.Blocks.Interfaces.RealOutput v_z(final quantity="Velocity", final unit="m/s")
          "Body vertical velocity";
          .Modelica.Blocks.Interfaces.RealOutput v_y(final quantity="Velocity", final unit="m/s")
          "Body lateral velocity";
          .Modelica.Blocks.Interfaces.RealOutput v_x(final quantity="Velocity", final unit="m/s")
          "Body longitudinal velocity";
          .Modelica.Blocks.Interfaces.RealOutput v_bat(final quantity="Voltage", final unit="V")
          "Battery voltage";
          .Modelica.Blocks.Interfaces.RealOutput v_t_y(final quantity="Velocity", final unit="m/s")
          "Target body lateral speed";
          .Modelica.Blocks.Interfaces.RealOutput v_t_x(final quantity="Velocity", final unit="m/s")
          "Target body longitudinal speed";
          .Modelica.Blocks.Interfaces.RealOutput s_t_y(final quantity="Position", final unit="m")
          "Target body lateral position";
          .Modelica.Blocks.Interfaces.RealOutput s_t_x(final quantity="Position", final unit="m")
          "Target body longitudinal position";
          WheelOutputs FL_wheel;
          WheelOutputs FR_wheel;
          WheelOutputs RL_wheel;
          WheelOutputs RR_wheel;
          annotation(Icon(
            coordinateSystem(preserveAspectRatio=true,
              extent={{-100.0,-100.0},{100.0,100.0}}),
              graphics={Polygon(lineColor={0,0,127},fillColor={255,255,255},fillPattern=FillPattern.Solid,points={{-100.0,100.0},{100.0,0.0},{-100.0,-100.0}}),Bitmap(fileName="modelica://VehicleDynamics/Resources/Images/Vehicles/Chassis/PackageIcon.png",origin={-24,3.000000000000003},extent={{-86,69},{86,-69}},rotation=180)}));
      end VehicleOutputs;

      connector WheelOutputs "Contact patch locations"
        .Modelica.Blocks.Interfaces.RealOutput i(final quantity="ElectricCurrent", final unit="A")
          "Wheel motor current";
        .Modelica.Blocks.Interfaces.RealOutput tau(final quantity="Torque", final unit="N.m")
          "Wheel motor torque";
        .Modelica.Blocks.Interfaces.RealOutput pow(final quantity="Power", final unit="W")
          "Wheel motor power";
        .Modelica.Blocks.Interfaces.RealOutput phi_str(final quantity="Angle", final unit="rad")
          "Tyre steering angle";
        .Modelica.Blocks.Interfaces.RealOutput f_z(final quantity="Force", final unit="N")
          "Tyre vertical force";
        .Modelica.Blocks.Interfaces.RealOutput f_y(final quantity="Force", final unit="N")
          "Tyre lateral force";
        .Modelica.Blocks.Interfaces.RealOutput f_x(final quantity="Force", final unit="N")
          "Tyre longitudinal force";
        .Modelica.Blocks.Interfaces.RealOutput omg(final quantity="AngularVelocity", final unit="rad/s");
        .Modelica.Blocks.Interfaces.RealOutput sr;
          annotation(Icon(
            coordinateSystem(preserveAspectRatio=true,
              extent={{-100.0,-100.0},{100.0,100.0}}),
              graphics={Polygon(lineColor={0,0,127},fillColor={255,255,255},fillPattern=FillPattern.Solid,points={{-100.0,100.0},{100.0,0.0},{-100.0,-100.0}}),Bitmap(fileName="modelica://VehicleDynamics/Resources/Images/Vehicles/Chassis/Wheels/PackageIcon.png",origin={-34,2},extent={{-74,67},{74,-67}},rotation=180)}));
      end WheelOutputs;

      connector DriverOutputs "Driver Outputs"
          .Modelica.Blocks.Interfaces.RealOutput str_cmd(final quantity="Angle", final unit="rad") "Closed loop driver steering command";
          .Modelica.Blocks.Interfaces.RealOutput brk_cmd "Closed loop driver brake command";
          .Modelica.Blocks.Interfaces.RealOutput acc_cmd "Closed loop driver accelerator command";

          .Modelica.Blocks.Interfaces.RealOutput vV_P_x[2](final quantity="Velocity", final unit="m/s") "Closed loop driver pathPoint reference velocity at t[k] t[k-1]";

          .Modelica.Blocks.Interfaces.RealOutput rV_x(final quantity="Length", final unit="m") "Closed loop driver pathPoint x position";
          .Modelica.Blocks.Interfaces.RealOutput rV_y(final quantity="Length", final unit="m") "Closed loop driver pathPoint y position";

          .Modelica.Blocks.Interfaces.RealOutput previewDistance[2](final quantity="Length", final unit="m") "Closed loop driver preview distance";

          .Modelica.Blocks.Interfaces.RealOutput previewTime[2](final quantity="Time", final unit="s") "Closed loop driver preview time";

          .Modelica.Blocks.Interfaces.RealOutput previewDistance_y(final quantity="Length", final unit="m") "Lateral tracker preview distance";

          .Modelica.Blocks.Interfaces.RealOutput previewTime_y(final quantity="Time", final unit="s") "Lateral tracker preview time";

          .Modelica.Blocks.Interfaces.RealOutput v_veh_percepts(final quantity="Velocity", final unit="m/s")
          "Closed loop driver percepted vehicle speed";
          .Modelica.Blocks.Interfaces.RealOutput vV_x_percepts(final quantity="Velocity", final unit="m/s")
          "Closed loop driver percepted vehicle speed in x";
          .Modelica.Blocks.Interfaces.RealOutput vV_y_percepts(final quantity="Velocity", final unit="m/s")
          "Closed loop driver percepted vehicle speed in y";
          .Modelica.Blocks.Interfaces.RealOutput vV_P_x_percepts(final quantity="Velocity", final unit="m/s")
          "Closed loop driver percepted vehicle speed along path in x";
          .Modelica.Blocks.Interfaces.RealOutput sG_y(final quantity="Position", final unit="m")
          "Target body lateral position";
          .Modelica.Blocks.Interfaces.RealOutput sG_x(final quantity="Position", final unit="m")
          "Target body longitudinal position";
          annotation(Icon(
            coordinateSystem(preserveAspectRatio=true,
              extent={{-100.0,-100.0},{100.0,100.0}}),
              graphics={Polygon(lineColor={0,0,127},fillColor={255,255,255},fillPattern=FillPattern.Solid,points={{-100.0,100.0},{100.0,0.0},{-100.0,-100.0}}),Bitmap(fileName="modelica://VehicleDynamics/Resources/Images/Drivers/StandardIcon.png",origin={-36,10},extent={{-68,51},{68,-51}},rotation=180)}));
      end DriverOutputs;

      connector EvaluationOutputs "Vehicle Outputs"
          .Modelica.Blocks.Interfaces.RealOutput J_v_1 "Integral of deviation from target velocity";
          .Modelica.Blocks.Interfaces.RealOutput J_v_2 "Integral of deviation from target path";
          .Modelica.Blocks.Interfaces.RealOutput J_e "Energy consumption";

          annotation(Icon(
            coordinateSystem(preserveAspectRatio=true,
              extent={{-100.0,-100.0},{100.0,100.0}}),
              graphics={Polygon(lineColor={0,0,127},fillColor={255,255,255},fillPattern=FillPattern.Solid,points={{-100.0,100.0},{100.0,0.0},{-100.0,-100.0}}),Line(origin={-40,23},points={{-36.20000000000002,-11.526075680404965},{36,11}}),Line(origin={-39,-11},points={{-1.500000000000007,33.272911914508015},{1,-33}}),Line(origin={-37.83201992511749,-43.854167103767395},points={{-39.83201992511749,-1.854167103767395},{39.83201992511749,1.854167103767395}}),Line(origin={-4,18},points={{-1.3000000000000114,16.373839952504454},{-10,-16},{10,-16},{0,16}}),Line(origin={-74,-4},points={{-1.3000000000000114,16.373839952504454},{-10,-16},{10,-16},{0,16}})}));
      end EvaluationOutputs;
    end Interfaces;

    package Chassis
        extends VehicleDynamics.Utilities.Icons.ChassisPackage;
      package Experiments
          extends Modelon.Icons.ExperimentsPackage;
        model FlatPad "Flat pad with compact chassis"
          extends .VehicleDynamics.Vehicles.Chassis.Experiments.Templates.FlatPad(
            zoom=1.1,redeclare replaceable .CDC2023.Vehicles.Chassis.Kinematic chassis(enable_inertial_states = false),
            redeclare replaceable .VehicleDynamics.Atmospheres.Constant atmosphere,fixedRig(enable_states = true,zPosition(s(start = chassis.h_start))));
            inner .Modelon.Mechanics.MultiBody.AggregateMass.AggregateMass aggregateMass(enable_aggregate = true,resolve = .Modelon.Mechanics.MultiBody.Internal.AggregateMass.Types.ResolveEnumeration.RESOLVE) annotation(Placement(transformation(extent = {{-100.0,80.0},{-80.0,100.0}},rotation = 0.0,origin = {0.0,0.0})));
        equation
            connect(aggregateMass.frame_resolve,chassis.vehicleFrame) annotation(Line(points = {{-90,80},{-90,-4},{18,-4},{18,-20}},color = {95,95,95},pattern = LinePattern.Dot));
        annotation (
            __Dymola_Commands(
              executeCall=
                .VehicleDynamics.Vehicles.Chassis.Experiments.Examples.Commands.visualize()
                "Visualize"),
           experiment(StopTime=10, __Dymola_Algorithm="Radau"),
           Documentation(info="<html>
<p>The flatpad simulation is a simulation of the chassis sitting stationary on a flat plate/ground. 
The chassis is constrained from moving laterally, longitudinally and in yaw by the fixedRig robot. 
The steering is also locked by inputting a constant zero steer input into the steerRobot.</p>
<p>This type of simulation is useful to look at the static balance of the car. This allows 
you to adjust spring preloads as necessary to trim the chassis due to either 
changing payloads or changes in spring rates. This is also a good simulation to use to evaluate 
the static toe and camber of the vehicle.</p>
<p>By changing the steering input to a different source, the steering effect on the chassis 
can also be characterized.</p>
</html>",         revisions="<html>Copyright &copy; 2004-2022, MODELON AB <br /> The use of this software component is regulated by the licensing conditions for Modelon Libraries. <br />This copyright notice must, unaltered, accompany all components that are derived from, copied from, <br />or by other means have their origin from any Modelon Library.</html>"));
        end FlatPad;

        model QuasiStatic
            extends .VehicleDynamics.Vehicles.Chassis.Experiments.Templates.QuasiStaticExternal(redeclare replaceable .CDC2023.Vehicles.Chassis.Kinematic chassis,robot(front_drive_proportion = 0.5));
            annotation(experiment(StartTime = 0,StopTime = 1,Interval = 0,__Dymola_Algorithm = "",Tolerance = 1e-6));
        end QuasiStatic;
      end Experiments;

      package Suspensions
          extends VehicleDynamics.Utilities.Icons.SuspensionsPackage;
        package Front
            extends Modelon.Icons.GroupingPackage;
          model Kinematic "Kinematic DoubleWishbone"
            extends .VehicleDynamics.Vehicles.Chassis.Suspensions.Front.DoubleWishbone(
              toe=0.005235987755983,
              camber=-0.0087266462599716,
              leftLinkage(
                r0H={0,0.75,0},
                r0CL1={0,0.37,-0.1},
                r0CL2={-0.31,0.37,-0.0},
                r0CL3={0.105,0.42,0.35},
                r0CL4={-0.105,0.42,0.34},
                r0CS={0.025,0.55,0.45},
                r0L1L2S={0,0.65,-0.08},
                r0L1L2U={-0.014,0.7,-0.1},
                r0L3L4U={-0.044,0.65,0.37},
                r0L5U={-0.136,0.7,-0.02},
                stabilizerMount(attachment_frame=3),
                steerAndUpperJoints(
                  upperArm(mass_calculation_method=1),
                  steerLink(mass_calculation_method=1),
                upperInnerJoints(visualize_forces_J1 = visualize_joint_forces,visualize_forces_J2 = visualize_joint_forces)),
                upright(
                  redeclare .VehicleDynamics.Vehicles.Chassis.Suspensions.Linkages.WheelCarriers.Components.Compliance hubMount(
                    enable_compliance=false,
                    k=[5.468e-07,-1.571e-07,0,-3.804e-07,-9.779e-07,-4.371e-07;
                      -1.565e-07,1.189e-07,0,1.883e-07,2.454e-07,8.019e-07;
                      -4.676e-13,3.782e-12,0,6.815e-13,5.079e-10,4.774e-11;
                      3.914e-07,-1.943e-07,0,-9.626e-07,-3.882e-07,-1.159e-06;
                      -9.772e-07,2.461e-07,0,3.771e-07,6.838e-06,4.658e-07;
                      -4.36e-07,8.023e-07,0,1.124e-06,4.643e-07,6.559e-06])),
                lowerInnerJoints(visualize_forces_J1 = visualize_joint_forces,visualize_forces_J2 = visualize_joint_forces)),
              rightLinkage(
                r0H=leftLinkage.r0H .* {1,-1,1},
                r0CL1=leftLinkage.r0CL1 .* {1,-1,1},
                r0CL2=leftLinkage.r0CL2 .* {1,-1,1},
                r0CL3=leftLinkage.r0CL3 .* {1,-1,1},
                r0CL4=leftLinkage.r0CL4 .* {1,-1,1},
                r0CS=leftLinkage.r0CS .* {1,-1,1},
                r0L1L2S=leftLinkage.r0L1L2S .* {1,-1,1},
                r0L1L2U=leftLinkage.r0L1L2U .* {1,-1,1},
                r0L3L4U=leftLinkage.r0L3L4U .* {1,-1,1},
                r0L5U=leftLinkage.r0L5U .* {1,-1,1},
                stabilizerMount(attachment_frame=3),
                steerAndUpperJoints(
                  upperArm(mass_calculation_method=1),
                  steerLink(mass_calculation_method=1),
                upperInnerJoints(visualize_forces_J1 = visualize_joint_forces,visualize_forces_J2 = visualize_joint_forces)),
                upright(
                  redeclare .VehicleDynamics.Vehicles.Chassis.Suspensions.Linkages.WheelCarriers.Components.Compliance hubMount(
                    enable_compliance=false,
                    k=[5.468e-07,1.563e-07,0,3.806e-07,-9.779e-07,4.365e-07;
                      1.565e-07,1.183e-07,0,1.884e-07,-2.454e-07,8.009e-07;
                      -4.658e-13,-4.074e-12,0,-6.294e-13,5.076e-10,-1.446e-10;
                      3.914e-07,1.933e-07,0,9.629e-07,-3.882e-07,1.158e-06;
                      -9.772e-07,-2.449e-07,0,-3.773e-07,6.838e-06,-4.65e-07;
                      4.36e-07,7.982e-07,0,1.125e-06,-4.643e-07,6.552e-06])),
                lowerInnerJoints(visualize_forces_J1 = visualize_joint_forces,visualize_forces_J2 = visualize_joint_forces)),
              steering(
                r0R_1={-0.191,0.395,-0.05},
                r0R_2=steering.r0R_1 .* {1,-1,1},
                iPR=-0.007,
                r0Q={-0.83,0.43,0.67}),
              stabilizer(
                r0LX_1={-0.056,0.544,-0.1},
                r0AL_1={-0.044,0.551,0.025},
                r0AC_1={-0.241,0.345,0.035},
                r0LX_2={-0.056,-0.544,-0.1},
                r0AL_2={-0.044,-0.551,0.025},
                r0AC_2={-0.241,-0.345,0.035}),
              leftSpring(
                s0=0,
                f0=4460,
                c=28300,
                c1=300000,
                s1=-0.08,
                c2=60000,
                s2=0.1,
                length1=0.01,
                length2=0.01),
              rightSpring(
                s0=0,
                f0=4460,
                c=28300,
                c1=300000,
                s1=-0.08,
                c2=60000,
                s2=0.1,
                length1=0.01,
                length2=0.01),
              leftDamper(d=3500),
              rightDamper(d=3500),
              stabilizerCompliance(c=3600));
              parameter Boolean visualize_joint_forces = false "True to visualize forces in joints" annotation(Dialog(tab = "Visualization"));
          end Kinematic;

          package Experiments
              extends Modelon.Icons.ExperimentsPackage;
            model TestSuspension
                extends .VehicleDynamics.Vehicles.Chassis.Suspensions.Experiments.Templates.ExternalRigControl.SteerableSuspensionInWheelRig(redeclare replaceable .CDC2023.Vehicles.Chassis.Suspensions.Front.Kinematic suspension(visualize_joint_forces = true));
            end TestSuspension;
          end Experiments;
        end Front;

        package Rear
            extends Modelon.Icons.GroupingPackage;
          model Kinematic "Kinematic DoubleWishbone"
            extends .VehicleDynamics.Vehicles.Chassis.Suspensions.Rear.DoubleWishbone(
              toe=0.005235987755983,
              camber=-0.026179938779915,
              leftLinkage(
                r0H={-2.7,0.75,0},
                r0CL1={-2.407,0.329,-0.03},
                r0CL2={-2.7,0.329,-0.13},
                r0CL3={-2.6,0.35,0.115},
                r0CL4={-2.8,0.35,0.115},
                r0CS={-2.7,0.541,0.325},
                r0L1L2S={-2.7,0.63,-0.11},
                r0L1L2U={-2.7,0.7,-0.13},
                r0L3L4U={-2.7,0.68,0.15},
                r0CL5={-2.55,0.33,-0.026},
                r0L5U={-2.55,0.663,0},
                stabilizerMount(attachment_frame=3),
                steerAndUpperJoints(
                  upperArm(mass_calculation_method=1),
                  steerLink(mass_calculation_method=1)),
                upright(
                  redeclare .VehicleDynamics.Vehicles.Chassis.Suspensions.Linkages.WheelCarriers.Components.Compliance hubMount(
                    enable_compliance=false,
                    k=[6.668e-07,9.246e-08,0,4.55e-07,4.704e-07,-5.625e-07;
                       9.243e-08,7.303e-08,0,1.409e-07,-3.744e-07,-5.115e-07;
                       -4.158e-12,-5.913e-12,0,6.325e-10,1.104e-11,5.39e-10;
                       4.54e-07,1.406e-07,0,3.381e-06,-2.527e-06,-9.846e-07;
                       4.71e-07,-3.749e-07,0,-2.533e-06,3.102e-05,4.825e-06;
                       -5.614e-07,-5.102e-07,0,-9.843e-07,4.802e-06,3.744e-06]))),
              rightLinkage(
                r0H=leftLinkage.r0H .* {1,-1,1},
                r0CL1=leftLinkage.r0CL1 .* {1,-1,1},
                r0CL2=leftLinkage.r0CL2 .* {1,-1,1},
                r0CL3=leftLinkage.r0CL3 .* {1,-1,1},
                r0CL4=leftLinkage.r0CL4 .* {1,-1,1},
                r0CS=leftLinkage.r0CS .* {1,-1,1},
                r0L1L2S=leftLinkage.r0L1L2S .* {1,-1,1},
                r0L1L2U=leftLinkage.r0L1L2U .* {1,-1,1},
                r0L3L4U=leftLinkage.r0L3L4U .* {1,-1,1},
                r0L5U=leftLinkage.r0L5U .* {1,-1,1},
                r0CL5=leftLinkage.r0CL5 .* {1,-1,1},
                stabilizerMount(attachment_frame=3),
                steerAndUpperJoints(
                  upperArm(mass_calculation_method=1),
                  steerLink(mass_calculation_method=1)),
                upright(
                  redeclare .VehicleDynamics.Vehicles.Chassis.Suspensions.Linkages.WheelCarriers.Components.Compliance hubMount(
                    enable_compliance=false,
                    k=[6.664e-07,-9.302e-08,0,-4.559e-07,4.711e-07,5.617e-07;
                       -9.238e-08,7.347e-08,0,1.412e-07,3.748e-07,-5.108e-07;
                       -3.411e-13,7.376e-12,0,8.784e-11,7.381e-11,1.059e-10;
                       -4.537e-07,1.415e-07,0,3.389e-06,2.53e-06,-9.831e-07;
                        4.708e-07,3.771e-07,0,2.538e-06,3.105e-05,-4.818e-06;
                        5.611e-07,-5.132e-07,0,-9.862e-07,-4.807e-06,3.74e-06]))),
              stabilizer(
                r0LX_1={-2.7,0.52,-0.1},
                r0AL_1={-2.705,0.52,0.025},
                r0AC_1={-2.941,0.345,0.035},
                r0LX_2={-2.7,-0.52,-0.1},
                r0AL_2={-2.705,-0.52,0.025},
                r0AC_2={-2.941,-0.345,0.035}),
              leftSpring(
                s0=0,
                f0=4185,
                c=64000,
                c1=300000,
                s1=-0.08,
                c2=60000,
                s2=0.1,
                length1=0.01,
                length2=0.01),
              rightSpring(
                s0=0,
                f0=4185,
                c=64000,
                c1=300000,
                s1=-0.08,
                c2=60000,
                s2=0.1,
                length1=0.01,
                length2=0.01),
              leftDamper(d=5000),
              rightDamper(d=5000),
              stabilizerCompliance(c=500));
          end Kinematic;

          package Experiments
              extends Modelon.Icons.ExperimentsPackage;
            model TestSuspension
                extends .VehicleDynamics.Vehicles.Chassis.Suspensions.Experiments.Templates.ExternalRigControl.SuspensionInWheelRig(redeclare replaceable .CDC2023.Vehicles.Chassis.Suspensions.Rear.Kinematic suspension);
            end TestSuspension;
          end Experiments;
        end Rear;
      end Suspensions;

      model Kinematic
        "Sedan chassis with kinematic suspensions and Pacejka02 tires"
        extends .VehicleDynamics.Vehicles.Chassis.Templates.Car(
          final icon_name="modelica://VehicleDynamics/Resources/Images/Vehicles/Chassis/SedanLKIcon.png",
          h_start=0.312,redeclare replaceable .CDC2023.Vehicles.Chassis.Suspensions.Front.Kinematic frontSuspension,redeclare replaceable .CDC2023.Vehicles.Chassis.Suspensions.Rear.Kinematic rearSuspension,
          redeclare .VehicleDynamics.Vehicles.Chassis.Bodies.Examples.Sedan body,
          redeclare replaceable .VehicleDynamics.Vehicles.Chassis.Wheels.Examples.Pacejka02.Conventional_205_55_R17 wheel_1(redeclare replaceable .VehicleDynamics.Vehicles.Chassis.Wheels.ContactCalculation.Generic contact(filtering = "Geometry weighted",nu = 5,phi_u = 0.78539816339744749917)),
          redeclare replaceable .VehicleDynamics.Vehicles.Chassis.Wheels.Examples.Pacejka02.Conventional_205_55_R17 wheel_2(redeclare replaceable .VehicleDynamics.Vehicles.Chassis.Wheels.ContactCalculation.Generic contact(filtering = "Geometry weighted",nu = 5,phi_u = 0.78539816339744749917)),
          redeclare replaceable .VehicleDynamics.Vehicles.Chassis.Wheels.Examples.Pacejka02.Conventional_205_55_R17 wheel_3(redeclare replaceable .VehicleDynamics.Vehicles.Chassis.Wheels.ContactCalculation.Generic contact(filtering = "Geometry weighted",nu = 5,phi_u = 0.78539816339744749917)),
          redeclare replaceable .VehicleDynamics.Vehicles.Chassis.Wheels.Examples.Pacejka02.Conventional_205_55_R17 wheel_4(redeclare replaceable .VehicleDynamics.Vehicles.Chassis.Wheels.ContactCalculation.Generic contact(filtering = "Geometry weighted",nu = 5,phi_u = 0.78539816339744749917)));

        annotation (
          Icon(coordinateSystem(
              preserveAspectRatio=false,
              extent={{-100,-50},{100,50}},
              grid={1,1}), graphics={
              Line(
                points={{-100,-30},{-21,-17}},
                color={0,0,0},
                pattern=LinePattern.Dot),
              Line(
                points={{-99,0},{-57,0}},
                color={0,0,0},
                pattern=LinePattern.Dot),
              Line(
                points={{100,-1},{60,-1}},
                color={0,0,0},
                pattern=LinePattern.Dot),
              Line(
                points={{-30,50},{20,0}},
                color={0,0,0},
                pattern=LinePattern.Dot),
              Polygon(
                points={{9,0},{14,1},{14,0},{20,0},{20,0},{14,0},{14,-1},{9,0}},
                lineColor={0,0,0},
                fillColor={0,0,0},
                fillPattern=FillPattern.Solid),
              Polygon(
                points={{20,0},{20,-6},{21,-6},{20,-10},{19,-6},{20,-6},{20,0},{20,0}},
                lineColor={0,0,0},
                fillColor={0,0,0},
                fillPattern=FillPattern.Solid),
              Ellipse(
                extent={{19,1},{21,-1}},
                lineColor={0,0,0},
                fillColor={0,0,0},
                fillPattern=FillPattern.Solid),
              Polygon(
                points={{-57,0},{-57,-6},{-56,-6},{-57,-10},{-58,-6},{-57,-6},{-57,0},
                    {-57,0}},
                lineColor={0,0,0},
                fillColor={0,0,0},
                fillPattern=FillPattern.Solid),
              Polygon(
                points={{-68,0},{-63,1},{-63,0},{-57,0},{-57,0},{-63,0},{-63,-1},{-68,
                    0}},
                lineColor={0,0,0},
                fillColor={0,0,0},
                fillPattern=FillPattern.Solid),
              Ellipse(
                extent={{-58,1},{-56,-1}},
                lineColor={0,0,0},
                fillColor={0,0,0},
                fillPattern=FillPattern.Solid),
              Polygon(
                points={{60,-1},{60,-7},{61,-7},{60,-11},{59,-7},{60,-7},{60,-1},{60,
                    -1}},
                lineColor={0,0,0},
                fillColor={0,0,0},
                fillPattern=FillPattern.Solid),
              Polygon(
                points={{49,-1},{54,0},{54,-1},{60,-1},{60,-1},{54,-1},{54,-2},{49,-1}},
                lineColor={0,0,0},
                fillColor={0,0,0},
                fillPattern=FillPattern.Solid),
              Ellipse(
                extent={{59,0},{61,-2}},
                lineColor={0,0,0},
                fillColor={0,0,0},
                fillPattern=FillPattern.Solid),
              Polygon(
                points={{-51,-29},{-51,-35},{-50,-35},{-51,-39},{-52,-35},{-51,-35},{
                    -51,-29},{-51,-29}},
                lineColor={0,0,0},
                fillColor={0,0,0},
                fillPattern=FillPattern.Solid),
              Polygon(
                points={{-62,-29},{-57,-28},{-57,-29},{-51,-29},{-51,-29},{-57,-29},{
                    -57,-30},{-62,-29}},
                lineColor={0,0,0},
                fillColor={0,0,0},
                fillPattern=FillPattern.Solid),
              Ellipse(
                extent={{-50,-28},{-52,-30}},
                lineColor={0,0,0},
                fillColor={0,0,0},
                fillPattern=FillPattern.Solid),
              Polygon(
                points={{-50,26},{-50,20},{-49,20},{-50,16},{-51,20},{-50,20},{-50,26},
                    {-50,26}},
                lineColor={0,0,0},
                fillColor={0,0,0},
                fillPattern=FillPattern.Solid),
              Polygon(
                points={{-61,26},{-56,27},{-56,26},{-50,26},{-50,26},{-56,26},{-56,25},
                    {-61,26}},
                lineColor={0,0,0},
                fillColor={0,0,0},
                fillPattern=FillPattern.Solid),
              Ellipse(
                extent={{-51,27},{-49,25}},
                lineColor={0,0,0},
                fillColor={0,0,0},
                fillPattern=FillPattern.Solid),
              Polygon(
                points={{46,26},{46,20},{47,20},{46,16},{45,20},{46,20},{46,26},{46,
                    26}},
                lineColor={0,0,0},
                fillColor={0,0,0},
                fillPattern=FillPattern.Solid),
              Polygon(
                points={{35,26},{40,27},{40,26},{46,26},{46,26},{40,26},{40,25},{35,
                    26}},
                lineColor={0,0,0},
                fillColor={0,0,0},
                fillPattern=FillPattern.Solid),
              Ellipse(
                extent={{45,27},{47,25}},
                lineColor={0,0,0},
                fillColor={0,0,0},
                fillPattern=FillPattern.Solid),
              Polygon(
                points={{47,-29},{47,-35},{48,-35},{47,-39},{46,-35},{47,-35},{47,-29},
                    {47,-29}},
                lineColor={0,0,0},
                fillColor={0,0,0},
                fillPattern=FillPattern.Solid),
              Polygon(
                points={{36,-29},{41,-28},{41,-29},{47,-29},{47,-29},{41,-29},{41,-30},
                    {36,-29}},
                lineColor={0,0,0},
                fillColor={0,0,0},
                fillPattern=FillPattern.Solid),
              Ellipse(
                extent={{46,-28},{48,-30}},
                lineColor={0,0,0},
                fillColor={0,0,0},
                fillPattern=FillPattern.Solid),
              Line(
                points={{-60,50},{-50,26}},
                color={0,0,0},
                pattern=LinePattern.Dot),
              Line(
                points={{-60,-50},{-50,-29}},
                color={0,0,0},
                pattern=LinePattern.Dot),
              Line(
                points={{60,-50},{47,-29}},
                color={0,0,0},
                pattern=LinePattern.Dot),
              Line(
                points={{60,50},{47,26}},
                color={0,0,0},
                pattern=LinePattern.Dot)}),
          Diagram(coordinateSystem(
              preserveAspectRatio=false,
              extent={{-100,-50},{100,50}},
              grid={1,1}), graphics),
          Documentation(info="<html>
<p>The model is an instantiation of 
<a href=\"modelica://VehicleDynamics.Vehicles.Chassis.Templates.Car#info\">Car</a> configured 
with elasto-kinematic suspension (double wishbone front and rear) and Pacejka02 wheel 
models. Parameters are set in each sub-component.
</p>
</html>",       revisions="<html>Copyright &copy; 2004-2022, MODELON AB <br /> The use of this software component is regulated by the licensing conditions for Modelon Libraries. <br />This copyright notice must, unaltered, accompany all components that are derived from, copied from, <br />or by other means have their origin from any Modelon Library.</html>"));
      end Kinematic;
    end Chassis;

    package Powertrains
        extends VehicleDynamics.Utilities.Icons.PowertrainsPackage;
      model InWheelDrive
        extends .VehicleDynamics.Vehicles.Powertrains.Templates.Electric.Detailed.InWheelDrive(
          icon_name="modelica://VehicleDynamics/Resources/Images/Vehicles/Powertrains/ElectricInWheelDriveIcon.png",
          minimumGear=min(gear_map[:, 2]),
          maximumGear=max(gear_map[:, 2]),
          gear_map=[-2,0,Integer(.VehicleDynamics.Vehicles.Powertrains.Transmissions.Types.GearSelectionMode.PARK);
                    -1,-1,Integer(.VehicleDynamics.Vehicles.Powertrains.Transmissions.Types.GearSelectionMode.REVERSE);
                    0,0,Integer(.VehicleDynamics.Vehicles.Powertrains.Transmissions.Types.GearSelectionMode.NEUTRAL);
                    1,1,Integer(.VehicleDynamics.Vehicles.Powertrains.Transmissions.Types.GearSelectionMode.DRIVE)],
          redeclare replaceable .Electrification.Batteries.Examples.MultiBody3D batteryPack(
            np=1,
            multibody3D(
              enable_mass_properties=false,
              m_pack=300,
              rcm=(r0H_1 + r0H_2 + r0H_3 + r0H_4)/4,
              r_shape=(r0H_1 + r0H_2 + r0H_3 + r0H_4)/4 + {-0.75,0,0},
              length=1.2,
              width=1.0,
              height=0.1),redeclare replaceable .CDC2023.Vehicles.Powertrains.BatteryController controller),
          redeclare replaceable .Electrification.Machines.Examples.Machine3D motor1(
            redeclare replaceable .Electrification.Machines.Core.Examples.Ideal core(
              controller_limits=true,
              limits(
                I_dc_max_mot=1e6,
                P_max_mot=35000,
                tau_max_mot=75)),
            redeclare .Electrification.Machines.Control.LimitedTorque controller(
              external_limits=true,
              listen=true,
              id_listen=batteryPack.id,
              typeListen=.Electrification.Utilities.Types.ControllerType.Battery,
              torqueControl(external_torque=true),
              k_pMax=1/4),
            multibody3D(
              enable_mass_properties=false,
              r_shape=r0H_1 + {0,-0.02,0},
              lengthDirection={0,-1,0},
              length=0.18,
              width=0.2),
            redeclare .Electrification.Machines.Mechanical.Gearbox mechanical(
              J_rotor=0,
              ratio=10,
              d_viscous=0)),
          redeclare replaceable .Electrification.Machines.Examples.Machine3D motor2(
            redeclare replaceable .Electrification.Machines.Core.Examples.Ideal core(
              controller_limits=true,
              limits(
                I_dc_max_mot=1e6,
                P_max_mot=35000,
                tau_max_mot=75)),
              thermal(fixed_temperature=true),
            redeclare .Electrification.Machines.Control.LimitedTorque controller(
              external_limits=true,
              listen=true,
              id_listen=batteryPack.id,
              typeListen=.Electrification.Utilities.Types.ControllerType.Battery,
              torqueControl(external_torque=true),
              k_pMax=1/4),
            multibody3D(
              enable_mass_properties=false,
              r_shape=r0H_2 + {0,0.02,0},
              lengthDirection={0,1,0},
              length=0.18,
              width=0.2),
            redeclare .Electrification.Machines.Mechanical.Gearbox mechanical(
              J_rotor=0,
              ratio=10,
              d_viscous=0)),
          redeclare replaceable .Electrification.Machines.Examples.Machine3D motor3(
            redeclare replaceable .Electrification.Machines.Core.Examples.Ideal core(
              controller_limits=true,
              limits(
                I_dc_max_mot=1e6,
                P_max_mot=35000,
                tau_max_mot=75)),
              thermal(fixed_temperature=true),
            redeclare .Electrification.Machines.Control.LimitedTorque controller(
              external_limits=true,
              listen=true,
              id_listen=batteryPack.id,
              typeListen=.Electrification.Utilities.Types.ControllerType.Battery,
              torqueControl(external_torque=true),
              k_pMax=1/4),
            multibody3D(
              enable_mass_properties=false,
              r_shape=r0H_3 + {0,-0.02,0},
              lengthDirection={0,-1,0},
              length=0.18,
              width=0.2),
            redeclare .Electrification.Machines.Mechanical.Gearbox mechanical(
              J_rotor=0,
              ratio=10,
              d_viscous=0)),
          redeclare replaceable .Electrification.Machines.Examples.Machine3D motor4(
            redeclare replaceable .Electrification.Machines.Core.Examples.Ideal core(
              controller_limits=true,
              limits(
                I_dc_max_mot=1e6,
                P_max_mot=35000,
                tau_max_mot=75)),
              thermal(fixed_temperature=true),
            redeclare .Electrification.Machines.Control.LimitedTorque controller(
              external_limits=true,
              listen=true,
              id_listen=batteryPack.id,
              typeListen=.Electrification.Utilities.Types.ControllerType.Battery,
              torqueControl(external_torque=true),
              k_pMax=1/4),
            multibody3D(
              enable_mass_properties=false,
              r_shape=r0H_4 + {0,0.02,0},
              lengthDirection={0,1,0},
              length=0.18,
              width=0.2),
            redeclare .Electrification.Machines.Mechanical.Gearbox mechanical(
              J_rotor=0,
              ratio=10,
              d_viscous=0)),redeclare replaceable .CDC2023.Vehicles.Powertrains.MotorControlFromBus controller,enable_accelerator = false,enable_shift = false);

        .Modelica.Units.SI.Torque summary_tau_1=-hub_1.flange.tau
          "Powertrain torque on left front";
        .Modelica.Units.SI.Torque summary_tau_2=-hub_2.flange.tau
          "Powertrain torque on right front";
        .Modelica.Units.SI.Torque summary_tau_3=-hub_3.flange.tau
          "Powertrain torque on left rear";
        .Modelica.Units.SI.Torque summary_tau_4=-hub_4.flange.tau
          "Powertrain torque on right rear";
                  .Electrification.Control.Interfaces.SystemBus controlBus annotation(Placement(transformation(extent = {{-26.411266944791766,43.58873305520823},{-13.588733055208234,56.41126694479177}},origin = {0.0,0.0},rotation = 0.0)));
      equation
          connect(controller.controlBus,controlBus) annotation(Line(points = {{22,30},{-20,30},{-20,50}},color = {240,170,40},pattern = LinePattern.Dot));
      end InWheelDrive;

      model MotorControlFromBus "Simple proportional throttle torque controller"
        extends .VehicleDynamics.Vehicles.Powertrains.ControlUnits.Electric.Interfaces.Base(final number_of_motors = 4,final k_throttle = 0,enable_shift = false);

        .Electrification.Machines.Control.Signals.tau_ref input_tau_ref[number_of_motors](
          id=1:number_of_motors)
          annotation (Placement(transformation(
              extent={{-6,-6},{6,6}},
              rotation=180,
              origin={-54,20})));
          .Modelon.Mechanics.Translational.Normalized.Fixed fixed annotation(Placement(transformation(extent = {{-70,76},{-50,96}},origin = {0,0},rotation = 0)));
          .Modelica.Blocks.Sources.Constant const(k = 2000) annotation(Placement(transformation(extent = {{6.0,-6.0},{-6.0,6.0}},rotation = 180.0,origin = {-36.0,-16.0})));
      equation
        for i in 1:number_of_motors loop
          connect(input_tau_ref[i].systemBus, controlBus) annotation (Line(
              points={{-60,20},{-100,20},{-100,0}},
              color={240,170,40},
              pattern=LinePattern.Dot,
              thickness=0.5), Text(
              string="%second",
              index=1,
              extent={{-6,3},{-6,3}},
              horizontalAlignment=TextAlignment.Right));
        end for;
        connect(input_tau_ref[1:number_of_motors].u_r,signalBus.VDL_trm_mtr_trq_cmd[1:number_of_motors]) annotation(Line(points = {{-45,20},{-22.5,20},{-22.5,80},{0,80}},color = {0,0,127}));
          connect(fixed.flange_a,acceleratorPedalFlange) annotation(Line(points = {{-60,86},{-60,100}},color = {0,127,0}));
          connect(const.y,signalBus.VDL_eng_spd_rpm) annotation(Line(points = {{-29.4,-16},{0,-16},{0,80}},color = {0,0,127}));

        annotation(Documentation(info="<html>
<p>
This is a simple motor torque controller for VDL powertrains using Electrification Library
components. The throttle input is multiplied by the <b>k_throttle</b> parameter (typically
the maximum motor torque) which is used as a torque command for each motor, with the number of
motors defined by the <b>number_of_motors</b> parameter.
</p>
<p>
Automatic transmission functionality is included using a <b>gear_map</b> for a single speed
transmission allowing the use of Parking, Reverse, Neutral and Drive modes.
</p>
</html>",       revisions="<html>Copyright &copy; 2004-2022, MODELON AB <br /> The use of this software component is regulated by the licensing conditions for Modelon Libraries. <br />This copyright notice must, unaltered, accompany all components that are derived from, copied from, <br />or by other means have their origin from any Modelon Library.</html>"));
      end MotorControlFromBus;

      model MinimalFourWheels
          extends .VehicleDynamics.Vehicles.Powertrains.Experiments.Templates.MinimalFourWheels(redeclare replaceable .CDC2023.Vehicles.Powertrains.InWheelDrive powertrain);
          .Modelica.Blocks.Sources.Trapezoid torqueSource[4](each rising = 0.5,each falling = 0.5,each width = 5,each startTime = 2,each offset = 0,each nperiod = 2,each amplitude = 100) annotation(Placement(transformation(extent = {{40.0,-132.0},{60.0,-112.0}},origin = {0.0,0.0},rotation = 0.0)));
          .CDC2023.Vehicles.Powertrains.i_sns i_sns_ annotation(Placement(transformation(extent = {{96.0,6.0},{88.0,14.0}},origin = {0.0,0.0},rotation = 0.0)));
      equation
          connect(torqueSource.y,signalBus.VDL_trm_mtr_trq_cmd) annotation(Line(points = {{61,-122},{75.5,-122},{75.5,-90},{90,-90}},color = {0,0,127}));
          connect(i_sns_.systemBus,powertrain.controlBus) annotation(Line(points = {{88,10},{44,10},{44,-20}},color = {240,170,40},pattern = LinePattern.Dot));
      end MinimalFourWheels;

      model i_sns "Sensed machine current"
          extends .Electrification.Machines.Control.Signals.Templates.SignalReal(causality = .Electrification.Utilities.Types.Causality.Output,unit = "A");
      equation
          connect(s,componentBus.i) annotation(Line(points = {{-24,0},{-2,0},{-2,0},{20,0}},color = {0,0,127}));
      end i_sns;

      model BatteryController
          extends .Electrification.Batteries.Control.LimitsFixed(
            SoC_max=1,
            SoC_high=0.9,
            SoC_low=0.1,
            SoC_min=0);
          .CDC2023.Vehicles.Powertrains.v_pack v_pack(causality = Electrification.Utilities.Types.Causality.Input) annotation(Placement(transformation(extent = {{48.0,58.0},{40.0,66.0}},origin = {0.0,0.0},rotation = 0.0)));
      equation
          connect(v_pack.u_r,electrical.core_v_sns) annotation(Line(points = {{50,62},{86,62},{86,-60},{100,-60}},color = {0,0,127}));
          connect(v_pack.systemBus,controlBus) annotation(Line(points = {{40,62},{20,62},{20,100},{0,100}},color = {240,170,40},pattern = LinePattern.Dot));
      end BatteryController;

      model v_pack "Cell voltage (min/max/avg)"
        extends .Electrification.Batteries.Control.Signals.Templates.SignalReal(
          causality=.Electrification.Utilities.Types.Causality.Output);

      equation
          connect(s,componentBus.v_pack) annotation(Line(points = {{-24,0},{-2,0},{-2,0},{20,0}},color = {0,0,127}));
      end v_pack;
    end Powertrains;

    model SedanInWheelMotors
        extends .VehicleDynamics.Vehicles.Templates.Car(final icon_name = "modelica://VehicleDynamics/Resources/Images/Vehicles/SedanLKIcon.png",redeclare replaceable .CDC2023.Vehicles.Chassis.Kinematic chassis,redeclare replaceable .CDC2023.Vehicles.Powertrains.InWheelDrive powertrain,redeclare replaceable .VehicleDynamics.Vehicles.Brakes.None brakes,redeclare replaceable .VehicleDynamics.Vehicles.Dashboards.Minimal dashboard,h_start = 0.312);
        .Electrification.Control.Interfaces.SystemBus controlBus annotation(Placement(transformation(extent = {{26.490898710197868,86.49089871019785},{53.50910128980213,113.50910128980215}},origin = {0.0,0.0},rotation = 0.0)));
    equation
        connect(powertrain.controlBus,controlBus) annotation(Line(points = {{120,-20},{120,74},{40,74},{40,100}},color = {240,170,40},pattern = LinePattern.Dot));
    end SedanInWheelMotors;

    package Experiments
        extends Modelon.Icons.ExperimentsPackage;
      model FlatPad
          extends .VehicleDynamics.Vehicles.Experiments.Templates.FlatPad(redeclare replaceable .CDC2023.Vehicles.SedanInWheelMotors vehicle,redeclare replaceable .VehicleDynamics.Atmospheres.Constant atmosphere,actuatedRig);
          .Modelica.Blocks.Sources.Constant torqueSource[4](each k=0) annotation(Placement(transformation(extent = {{-70.0,-68.0},{-50.0,-48.0}},origin = {0.0,0.0},rotation = 0.0)));
          .VehicleDynamics.Vehicles.Interfaces.SignalBus signalBus annotation(Placement(transformation(extent = {{-48.0,-58.0},{-28.0,-38.0}},origin = {0.0,0.0},rotation = 0.0)));
        inner Modelon.Mechanics.MultiBody.AggregateMass.AggregateMass
          aggregateMass(resolve=Modelon.Mechanics.MultiBody.Internal.AggregateMass.Types.ResolveEnumeration.RESOLVE,
            enable_aggregate=true)
          annotation (Placement(transformation(extent={{-100,80},{-80,100}})));
      equation
          connect(torqueSource.y,signalBus.VDL_trm_mtr_trq_cmd) annotation(Line(points = {{-49,-58},{-38,-58},{-38,-48}},color = {0,0,127}));
          connect(vehicle.signalBus,signalBus) annotation(Line(points = {{-20,-40},{-29,-40},{-29,-48},{-38,-48}},color = {255,204,51}));
        connect(aggregateMass.frame_resolve, vehicle.vehicleFrame) annotation (
            Line(
            points={{-90,80},{-90,44},{-66,44},{-66,34},{-58,34},{-58,-10},{-26,-10},
                {-26,-25},{-20,-25}},
            color={95,95,95},
            pattern=LinePattern.Dot));
      end FlatPad;

      model Minimal
          extends .VehicleDynamics.Vehicles.Experiments.Templates.Minimal(redeclare replaceable .VehicleDynamics.Atmospheres.Constant atmosphere,redeclare replaceable .VehicleDynamics.Grounds.Flat ground,redeclare replaceable .CDC2023.Vehicles.SedanInWheelMotors vehicle(v_start = 0));
          replaceable .Modelica.Blocks.Sources.Trapezoid torqueSource[4](amplitude = 100,rising = 0.5,width = 5,falling = 0.5,period = 8,nperiod = -1,offset = 0,startTime = 5) constrainedby .Modelica.Blocks.Interfaces.SO annotation(choicesAllMatching,Placement(transformation(extent = {{-107.0,-45.0},{-87.0,-25.0}},origin = {0.0,0.0},rotation = 0.0)));
          .VehicleDynamics.Vehicles.Interfaces.SignalBus signalBus annotation(Placement(transformation(extent = {{-85.0,-35.0},{-65.0,-15.0}},origin = {0.0,0.0},rotation = 0.0)));
          replaceable .Modelica.Blocks.Sources.Constant steerSource(k = 0) constrainedby .Modelica.Blocks.Interfaces.SO annotation(choicesAllMatching,Placement(transformation(extent = {{-124.0,1.3333333333333357},{-104.0,21.333333333333336}},rotation = 0.0,origin = {0.0,0.0})));
          .VehicleDynamics.Vehicles.Chassis.Experiments.Robots.SteerServo steerRobot "Robot for steering" annotation(Placement(transformation(extent = {{-64.0,1.3333333333333357},{-44.0,21.333333333333336}},rotation = 0.0,origin = {0.0,0.0})));
          .Modelica.Blocks.Sources.BooleanConstant release(k = false) annotation(Placement(transformation(extent = {{-94.0,21.333333333333336},{-74.0,41.333333333333336}},rotation = 0.0,origin = {0.0,0.0})));
      equation
          connect(torqueSource.y,signalBus.VDL_trm_mtr_trq_cmd) annotation(Line(points = {{-86,-35},{-75,-35},{-75,-25}},color = {0,0,127}));
          connect(vehicle.signalBus,signalBus) annotation(Line(points = {{-60,-70},{-75,-70},{-75,-25}},color = {255,204,51}));
          connect(steerSource.y,steerRobot.phi_ref) annotation(Line(points = {{-103,11.333333333333336},{-65,11.333333333333336}},color = {0,0,127}));
          connect(release.y,steerRobot.release) annotation(Line(points = {{-73,31.333333333333336},{-54,31.333333333333336},{-54,22.333333333333336}},color = {255,0,255}));
          connect(vehicle.steeringWheelFlange,steerRobot.steeringWheelFlange) annotation(Line(points = {{-35,-30},{-35,11.333333333333336},{-44,11.333333333333336}},color = {95,95,95}));
      end Minimal;

      model SimplePathFollowing
          extends .VehicleDynamics.Vehicles.Experiments.Templates.Minimal(redeclare replaceable .VehicleDynamics.Atmospheres.Constant atmosphere,redeclare replaceable .VehicleDynamics.Grounds.OpenCRG ground(file_name = Modelica.Utilities.Files.loadResource("modelica://VehicleDynamics/Resources/Data/Roads/crg-txt/flat_250.crg"),override_velocity = false,override_offset = false,path_file = Modelica.Utilities.Files.loadResource("modelica://VehicleDynamics/Resources/Data/Roads/crg-pth/doubleLaneChange.mat")),redeclare replaceable .CDC2023.Vehicles.SedanInWheelMotors vehicle(v_start = 0,r_y_start = -4));
          replaceable .Modelica.Blocks.Sources.Trapezoid torqueSource[4](amplitude = 50,rising = 0.5,width = 5,falling = 0.5,period = 8,nperiod = 1,offset = 0,startTime = 5) constrainedby .Modelica.Blocks.Interfaces.SO annotation(choicesAllMatching,Placement(transformation(extent = {{-107.0,-45.0},{-87.0,-25.0}},origin = {0.0,0.0},rotation = 0.0)));
          .VehicleDynamics.Vehicles.Interfaces.SignalBus signalBus annotation(Placement(transformation(extent = {{-85.0,-35.0},{-65.0,-15.0}},origin = {0.0,0.0},rotation = 0.0)));
          replaceable .Modelica.Blocks.Sources.Constant steerSource(k = 0) constrainedby .Modelica.Blocks.Interfaces.SO annotation(choicesAllMatching,Placement(transformation(extent = {{-124.0,1.3333333333333357},{-104.0,21.333333333333336}},rotation = 0.0,origin = {0.0,0.0})));
          .VehicleDynamics.Vehicles.Chassis.Experiments.Robots.SteerServo steerRobot "Robot for steering" annotation(Placement(transformation(extent = {{-64.0,1.3333333333333357},{-44.0,21.333333333333336}},rotation = 0.0,origin = {0.0,0.0})));
          .Modelica.Blocks.Sources.BooleanConstant release(k = false) annotation(Placement(transformation(extent = {{-94.0,21.333333333333336},{-74.0,41.333333333333336}},rotation = 0.0,origin = {0.0,0.0})));
          .VehicleDynamics.Drivers.Perception.Ideal perception annotation(Placement(transformation(extent = {{-25.0,16.0},{-5.0,36.0}},origin = {0.0,0.0},rotation = 0.0)));
          .VehicleDynamics.Drivers.Planning.FromGround planning annotation(Placement(transformation(extent = {{9.0,16.0},{29.0,36.0}},origin = {0.0,0.0},rotation = 0.0)));
          replaceable .Modelica.Blocks.Sources.Constant previewTime(k = 0) constrainedby .Modelica.Blocks.Interfaces.SO annotation(choicesAllMatching,Placement(transformation(extent = {{-20.0,58.0},{0.0,78.0}},rotation = 0.0,origin = {0.0,0.0})));
          replaceable .Modelica.Blocks.Sources.Constant previewDistance(k = 10) constrainedby .Modelica.Blocks.Interfaces.SO annotation(choicesAllMatching,Placement(transformation(extent = {{6.0,84.0},{26.0,104.0}},rotation = 0.0,origin = {0.0,0.0})));
          .VehicleDynamics.Drivers.Tracking.Blocks.SinglePointLateralTracker lateralTracker annotation(Placement(transformation(extent = {{44.0,12.0},{64.0,32.0}},origin = {0.0,0.0},rotation = 0.0)));
      equation
          connect(torqueSource.y,signalBus.VDL_trm_mtr_trq_cmd) annotation(Line(points = {{-86,-35},{-75,-35},{-75,-25}},color = {0,0,127}));
          connect(vehicle.signalBus,signalBus) annotation(Line(points = {{-60,-70},{-75,-70},{-75,-25}},color = {255,204,51}));
          connect(release.y,steerRobot.release) annotation(Line(points = {{-73,31.333333333333336},{-54,31.333333333333336},{-54,22.333333333333336}},color = {255,0,255}));
          connect(vehicle.steeringWheelFlange,steerRobot.steeringWheelFlange) annotation(Line(points = {{-35,-30},{-35,11.333333333333336},{-44,11.333333333333336}},color = {95,95,95}));
          connect(perception.perceptionFrame,vehicle.vehicleFrame) annotation(Line(points = {{-15,16},{-15,10},{-66,10},{-66,-55},{-60,-55}},color = {95,95,95}));
          connect(perception.dashboardInformation,signalBus.dashboard) annotation(Line(points = {{-15,36},{-15,42},{-75,42},{-75,-25}},color = {0,0,0}));
          connect(perception.percepts,planning.percepts) annotation(Line(points = {{-3,26},{7,26}},color = {0,128,255},pattern = LinePattern.Dash));
          connect(previewTime.y,planning.previewTime[1]) annotation(Line(points = {{1,68},{15,68},{15,37}},color = {0,0,127}));
          connect(previewDistance.y,planning.previewDistance[1]) annotation(Line(points = {{27,94},{23,94},{23,37}},color = {0,0,127}));
          connect(planning.pathPoint,lateralTracker.pathPoint) annotation(Line(points = {{31,26},{42,26}},color = {255,0,0}));
          connect(lateralTracker.percepts,perception.percepts) annotation(Line(points = {{42,18},{34,18},{34,10},{2,10},{2,26},{-3,26}},color = {0,128,255},pattern = LinePattern.Dash));
          connect(lateralTracker.str_cmd,steerRobot.phi_ref) annotation(Line(points = {{65,22},{71,22},{71,-6},{-70,-6},{-70,11.333333333333336},{-65,11.333333333333336}},color = {0,0,127}));
      end SimplePathFollowing;

      model MaxAcceleration
          extends .VehicleDynamics.Vehicles.Experiments.Templates.Minimal(redeclare replaceable .VehicleDynamics.Atmospheres.Constant atmosphere,redeclare replaceable .VehicleDynamics.Grounds.Flat ground,redeclare replaceable .CDC2023.Vehicles.SedanInWheelMotors vehicle(v_start = 0));
          replaceable .Modelica.Blocks.Sources.Ramp torqueSource[4](height = {75,75,75,75},duration = {0.1,0.1,0.1,0.1},startTime = {5,5,5,5}) constrainedby .Modelica.Blocks.Interfaces.SO annotation(choicesAllMatching,Placement(transformation(extent = {{-107.0,-45.0},{-87.0,-25.0}},origin = {0.0,0.0},rotation = 0.0)));
          .VehicleDynamics.Vehicles.Interfaces.SignalBus signalBus annotation(Placement(transformation(extent = {{-85.0,-35.0},{-65.0,-15.0}},origin = {0.0,0.0},rotation = 0.0)));
          replaceable .Modelica.Blocks.Sources.Constant steerSource(k = 0) constrainedby .Modelica.Blocks.Interfaces.SO annotation(choicesAllMatching,Placement(transformation(extent = {{-124.0,1.3333333333333357},{-104.0,21.333333333333336}},rotation = 0.0,origin = {0.0,0.0})));
          .VehicleDynamics.Vehicles.Chassis.Experiments.Robots.SteerServo steerRobot "Robot for steering" annotation(Placement(transformation(extent = {{-64.0,1.3333333333333357},{-44.0,21.333333333333336}},rotation = 0.0,origin = {0.0,0.0})));
          .Modelica.Blocks.Sources.BooleanConstant release(k = false) annotation(Placement(transformation(extent = {{-94.0,21.333333333333336},{-74.0,41.333333333333336}},rotation = 0.0,origin = {0.0,0.0})));
      equation
          connect(torqueSource.y,signalBus.VDL_trm_mtr_trq_cmd) annotation(Line(points = {{-86,-35},{-75,-35},{-75,-25}},color = {0,0,127}));
          connect(vehicle.signalBus,signalBus) annotation(Line(points = {{-60,-70},{-75,-70},{-75,-25}},color = {255,204,51}));
          connect(steerSource.y,steerRobot.phi_ref) annotation(Line(points = {{-103,11.333333333333336},{-65,11.333333333333336}},color = {0,0,127}));
          connect(release.y,steerRobot.release) annotation(Line(points = {{-73,31.333333333333336},{-54,31.333333333333336},{-54,22.333333333333336}},color = {255,0,255}));
          connect(vehicle.steeringWheelFlange,steerRobot.steeringWheelFlange) annotation(Line(points = {{-35,-30},{-35,11.333333333333336},{-44,11.333333333333336}},color = {95,95,95}));
      end MaxAcceleration;
    end Experiments;

    package Sensors
        extends Modelon.Icons.SensorsPackage;
      model SensorPackage
          extends Modelon.Icons.MultipleSensors;
          .Modelica.Mechanics.MultiBody.Interfaces.Frame_a frame_a annotation(Placement(transformation(extent = {{-116,-16},{-84,16}},origin = {0,0},rotation = 0)));

          .Modelon.Mechanics.Rotational3D.Interfaces.Flange_a hub_1 "To be connected to the wheel hub flange" annotation(Placement(transformation(extent = {{-80.0,-110.0},{-60.0,-90.0}},rotation = 0.0,origin = {0.0,0.0})));
          .Modelon.Mechanics.Rotational3D.Interfaces.Flange_a hub_2 "To be connected to the wheel hub flange" annotation(Placement(transformation(extent = {{-40.0,-110.0},{-20.0,-90.0}},rotation = 0.0,origin = {0.0,0.0})));
          .Modelon.Mechanics.Rotational3D.Interfaces.Flange_a hub_3 "To be connected to the wheel hub flange" annotation(Placement(transformation(extent = {{20.0,-110.0},{40.0,-90.0}},rotation = 0.0,origin = {0.0,0.0})));
          .Modelon.Mechanics.Rotational3D.Interfaces.Flange_a hub_4 "To be connected to the wheel hub flange" annotation(Placement(transformation(extent = {{60.0,-110.0},{80.0,-90.0}},rotation = 0.0,origin = {0.0,0.0})));

          .VehicleDynamics.Vehicles.Chassis.Sensors.HubAngles hubAngles_1 annotation(Placement(transformation(extent = {{-70.0,-80.0},{-50.0,-60.0}},origin = {0.0,0.0},rotation = 0.0)));
          .VehicleDynamics.Vehicles.Chassis.Sensors.HubAngles hubAngles_2 annotation(Placement(transformation(extent = {{-30.0,-80.0},{-10.0,-60.0}},origin = {0.0,0.0},rotation = 0.0)));
          .VehicleDynamics.Vehicles.Chassis.Sensors.HubAngles hubAngles_3 annotation(Placement(transformation(extent = {{30.0,-80.0},{10.0,-60.0}},origin = {0.0,0.0},rotation = 0.0)));
          .VehicleDynamics.Vehicles.Chassis.Sensors.HubAngles hubAngles_4 annotation(Placement(transformation(extent = {{70.0,-80.0},{50.0,-60.0}},origin = {0.0,0.0},rotation = 0.0)));

          .VehicleDynamics.Vehicles.Interfaces.SignalBus signalBus annotation(Placement(transformation(extent = {{-119.76757610476429,40.23242389523571},{-80.23242389523571,79.76757610476429}},origin = {0.0,0.0},rotation = 0.0)));
          .CDC2023.Vehicles.Interfaces.VehicleOutputs vehicleOutputs annotation(Placement(transformation(extent = {{90.0,-10.0},{110.0,10.0}},origin = {0.0,0.0},rotation = 0.0)));
          .Electrification.Control.Interfaces.SystemBus controlBus annotation(Placement(transformation(extent = {{-120,-80},{-80,-40}},origin = {0,0},rotation = 0)));

          .Electrification.Machines.Control.Signals.tau_sns[4] tau_sns(id = {1,2,3,4}) annotation(Placement(transformation(extent = {{-46.0,-4.0},{-54.0,4.0}},origin = {0.0,0.0},rotation = 0.0)));
          .Electrification.Machines.Control.Signals.pwr_sns[4] pwr_sns(id = {1,2,3,4}) annotation(Placement(transformation(extent = {{-46.0,-14.0},{-54.0,-6.0}},origin = {0.0,0.0},rotation = 0.0)));
          .CDC2023.Vehicles.Powertrains.i_sns[4] i_sns annotation(Placement(transformation(extent = {{-46.0,-24.0},{-54.0,-16.0}},origin = {0.0,0.0},rotation = 0.0)));

          .VehicleDynamics.Vehicles.Chassis.Sensors.ChassisStates chassisStates annotation(Placement(transformation(extent = {{-20,30},{0,50}},origin = {0,0},rotation = 0)));
          .CDC2023.Vehicles.Powertrains.v_pack v_pack annotation(Placement(transformation(extent = {{-4.0,4.0},{-12.0,12.0}},origin = {0.0,0.0},rotation = 0.0)));
          .CDC2023.Vehicles.Interfaces.DriverOutputs driverOutputs annotation(Placement(transformation(extent = {{-10.0,-10.0},{10.0,10.0}},origin = {0.0,110.0},rotation = -90.0)));
      protected
          .CDC2023.Vehicles.Interfaces.WheelOutputs[4] wheelOutputs annotation(Placement(transformation(extent = {{60.0,-34.0},{80.0,-14.0}},origin = {0.0,0.0},rotation = 0.0)));
      equation
          connect(chassisStates.mount_frame,frame_a) annotation(Line(points = {{-20,40},{-60,40},{-60,0},{-100,0}},color = {95,95,95}));
          connect(chassisStates.velocity[3],vehicleOutputs.v_z) annotation(Line(points = {{1,44},{76,44},{76,0},{100,0}},color = {0,0,127}));
          connect(chassisStates.velocity[1],vehicleOutputs.v_x) annotation(Line(points = {{1,44},{80,44},{80,0},{100,0}},color = {0,0,127}));
          connect(chassisStates.velocity[2],vehicleOutputs.v_y) annotation(Line(points = {{1,44},{78,44},{78,0},{100,0}},color = {0,0,127}));
          connect(chassisStates.acceleration[1],vehicleOutputs.a_x) annotation(Line(points = {{1,41},{56,41},{56,0},{100,0}},color = {0,0,127}));
          connect(chassisStates.acceleration[2],vehicleOutputs.a_y) annotation(Line(points = {{1,41},{54,41},{54,0},{100,0}},color = {0,0,127}));
          connect(chassisStates.acceleration[3],vehicleOutputs.a_z) annotation(Line(points = {{1,41},{52,41},{52,0},{100,0}},color = {0,0,127}));
          connect(chassisStates.angles[1],vehicleOutputs.phi_p) annotation(Line(points = {{1,38},{44,38},{44,0},{100,0}},color = {0,0,127}));
          connect(chassisStates.angles[2],vehicleOutputs.phi_q) annotation(Line(points = {{1,38},{42,38},{42,0},{100,0}},color = {0,0,127}));
          connect(chassisStates.angles[3],vehicleOutputs.phi_r) annotation(Line(points = {{1,38},{40,38},{40,0},{100,0}},color = {0,0,127}));
          connect(chassisStates.angular_velocity[1],vehicleOutputs.omg_p) annotation(Line(points = {{1,35},{30,35},{30,0},{100,0}},color = {0,0,127}));
          connect(chassisStates.angular_velocity[2],vehicleOutputs.omg_q) annotation(Line(points = {{1,35},{28,35},{28,0},{100,0}},color = {0,0,127}));
          connect(chassisStates.angular_velocity[3],vehicleOutputs.omg_r) annotation(Line(points = {{1,35},{26,35},{26,0},{100,0}},color = {0,0,127}));
          connect(tau_sns[1].systemBus,controlBus) annotation(Line(points = {{-54,0},{-75,0},{-75,-60},{-100,-60}},color = {240,170,40},pattern = LinePattern.Dot));
          connect(tau_sns[2].systemBus,controlBus) annotation(Line(points = {{-54,0},{-75,0},{-75,-60},{-100,-60}},color = {240,170,40},pattern = LinePattern.Dot));
          connect(tau_sns[3].systemBus,controlBus) annotation(Line(points = {{-54,0},{-75,0},{-75,-60},{-100,-60}},color = {240,170,40},pattern = LinePattern.Dot));
          connect(tau_sns[4].systemBus,controlBus) annotation(Line(points = {{-54,0},{-75,0},{-75,-60},{-100,-60}},color = {240,170,40},pattern = LinePattern.Dot));
          connect(pwr_sns[1].systemBus,controlBus) annotation(Line(points = {{-54,-10},{-75,-10},{-75,-60},{-100,-60}},color = {240,170,40},pattern = LinePattern.Dot));
          connect(pwr_sns[2].systemBus,controlBus) annotation(Line(points = {{-54,-10},{-75,-10},{-75,-60},{-100,-60}},color = {240,170,40},pattern = LinePattern.Dot));
          connect(pwr_sns[3].systemBus,controlBus) annotation(Line(points = {{-54,-10},{-75,-10},{-75,-60},{-100,-60}},color = {240,170,40},pattern = LinePattern.Dot));
          connect(pwr_sns[4].systemBus,controlBus) annotation(Line(points = {{-54,-10},{-75,-10},{-75,-60},{-100,-60}},color = {240,170,40},pattern = LinePattern.Dot));
          connect(i_sns[1].systemBus,controlBus) annotation(Line(points = {{-54,-20},{-54,-60},{-100,-60}},color = {240,170,40},pattern = LinePattern.Dot));
          connect(i_sns[2].systemBus,controlBus) annotation(Line(points = {{-54,-20},{-54,-60},{-100,-60}},color = {240,170,40},pattern = LinePattern.Dot));
          connect(i_sns[3].systemBus,controlBus) annotation(Line(points = {{-54,-20},{-54,-60},{-100,-60}},color = {240,170,40},pattern = LinePattern.Dot));
          connect(i_sns[4].systemBus,controlBus) annotation(Line(points = {{-54,-20},{-54,-60},{-100,-60}},color = {240,170,40},pattern = LinePattern.Dot));
          connect(tau_sns[1].y_r,wheelOutputs[1].tau) annotation(Line(points = {{-45,0},{-19.5,0},{-19.5,-24},{70,-24}},color = {0,0,127}));
          connect(tau_sns[2].y_r,wheelOutputs[2].tau) annotation(Line(points = {{-45,0},{-19.5,0},{-19.5,-24},{70,-24}},color = {0,0,127}));
          connect(tau_sns[3].y_r,wheelOutputs[3].tau) annotation(Line(points = {{-45,0},{-19.5,0},{-19.5,-24},{70,-24}},color = {0,0,127}));
          connect(tau_sns[4].y_r,wheelOutputs[4].tau) annotation(Line(points = {{-45,0},{-19.5,0},{-19.5,-24},{70,-24}},color = {0,0,127}));
          connect(pwr_sns[1].y_r,wheelOutputs[1].pow) annotation(Line(points = {{-45,-10},{-19.5,-10},{-19.5,-24},{70,-24}},color = {0,0,127}));
          connect(pwr_sns[2].y_r,wheelOutputs[2].pow) annotation(Line(points = {{-45,-10},{-19.5,-10},{-19.5,-24},{70,-24}},color = {0,0,127}));
          connect(pwr_sns[3].y_r,wheelOutputs[3].pow) annotation(Line(points = {{-45,-10},{-19.5,-10},{-19.5,-24},{70,-24}},color = {0,0,127}));
          connect(pwr_sns[4].y_r,wheelOutputs[4].pow) annotation(Line(points = {{-45,-10},{-19.5,-10},{-19.5,-24},{70,-24}},color = {0,0,127}));
          connect(i_sns[1].y_r,wheelOutputs[1].i) annotation(Line(points = {{-45,-20},{-19.5,-20},{-19.5,-24},{70,-24}},color = {0,0,127}));
          connect(i_sns[2].y_r,wheelOutputs[2].i) annotation(Line(points = {{-45,-20},{-19.5,-20},{-19.5,-24},{70,-24}},color = {0,0,127}));
          connect(i_sns[3].y_r,wheelOutputs[3].i) annotation(Line(points = {{-45,-20},{-19.5,-20},{-19.5,-24},{70,-24}},color = {0,0,127}));
          connect(i_sns[4].y_r,wheelOutputs[4].i) annotation(Line(points = {{-45,-20},{-19.5,-20},{-19.5,-24},{70,-24}},color = {0,0,127}));
          connect(wheelOutputs[1],vehicleOutputs.FL_wheel) annotation(Line(points = {{70,-24},{88,-24},{88,0},{100,0}},color = {0,0,0}));
          connect(wheelOutputs[2],vehicleOutputs.FR_wheel) annotation(Line(points = {{70,-24},{86,-24},{86,0},{100,0}},color = {0,0,0}));
          connect(wheelOutputs[3],vehicleOutputs.RL_wheel) annotation(Line(points = {{70,-24},{84,-24},{84,0},{100,0}},color = {0,0,0}));
          connect(wheelOutputs[4],vehicleOutputs.RR_wheel) annotation(Line(points = {{70,-24},{82,-24},{82,0},{100,0}},color = {0,0,0}));
          connect(vehicleOutputs.FL_wheel.f_x,signalBus.VDL_whl_frc_x[1]) annotation(Line(points = {{100,0},{88,0},{88,60},{-100,60}},color = {0,0,0}));
          connect(vehicleOutputs.FR_wheel.f_x,signalBus.VDL_whl_frc_x[2]) annotation(Line(points = {{100,0},{90,0},{90,60},{-100,60}},color = {0,0,0}));
          connect(vehicleOutputs.RL_wheel.f_x,signalBus.VDL_whl_frc_x[3]) annotation(Line(points = {{100,0},{90,0},{90,60},{-100,60}},color = {0,0,0}));
          connect(vehicleOutputs.RR_wheel.f_x,signalBus.VDL_whl_frc_x[4]) annotation(Line(points = {{100,0},{90,0},{90,60},{-100,60}},color = {0,0,0}));
          connect(vehicleOutputs.FL_wheel.f_y,signalBus.VDL_whl_frc_y[1]) annotation(Line(points = {{100,0},{90,0},{90,60},{-100,60}},color = {0,0,0}));
          connect(vehicleOutputs.FR_wheel.f_y,signalBus.VDL_whl_frc_y[2]) annotation(Line(points = {{100,0},{90,0},{90,60},{-100,60}},color = {0,0,0}));
          connect(vehicleOutputs.RL_wheel.f_y,signalBus.VDL_whl_frc_y[3]) annotation(Line(points = {{100,0},{90,0},{90,60},{-100,60}},color = {0,0,0}));
          connect(vehicleOutputs.RR_wheel.f_y,signalBus.VDL_whl_frc_y[4]) annotation(Line(points = {{100,0},{90,0},{90,60},{-100,60}},color = {0,0,0}));
          connect(vehicleOutputs.FL_wheel.f_z,signalBus.VDL_whl_frc_z[1]) annotation(Line(points = {{100,0},{90,0},{90,60},{-100,60}},color = {0,0,0}));
          connect(vehicleOutputs.FR_wheel.f_z,signalBus.VDL_whl_frc_z[2]) annotation(Line(points = {{100,0},{88,0},{88,60},{-100,60}},color = {0,0,0}));
          connect(vehicleOutputs.RL_wheel.f_z,signalBus.VDL_whl_frc_z[3]) annotation(Line(points = {{100,0},{88,0},{88,60},{-100,60}},color = {0,0,0}));
          connect(vehicleOutputs.RR_wheel.f_z,signalBus.VDL_whl_frc_z[4]) annotation(Line(points = {{100,0},{90,0},{90,60},{-100,60}},color = {0,0,0}));
          connect(hubAngles_1.frame_resolve,frame_a) annotation(Line(points = {{-60,-80},{-60,-84},{-100,-84},{-100,0}},color = {95,95,95},pattern = LinePattern.Dot));
          connect(hubAngles_1.hubFlange,hub_1) annotation(Line(points = {{-70,-70},{-70,-100}},color = {95,95,95}));
          connect(hubAngles_2.frame_resolve,frame_a) annotation(Line(points = {{-20,-80},{-20,-84},{-100,-84},{-100,0}},color = {95,95,95},pattern = LinePattern.Dot));
          connect(hubAngles_3.frame_resolve,frame_a) annotation(Line(points = {{20,-80},{20,-86},{-100,-86},{-100,0}},color = {95,95,95},pattern = LinePattern.Dot));
          connect(hubAngles_4.frame_resolve,frame_a) annotation(Line(points = {{60,-80},{60,-86},{-100,-86},{-100,0}},color = {95,95,95},pattern = LinePattern.Dot));
          connect(hubAngles_2.hubFlange,hub_2) annotation(Line(points = {{-30,-70},{-30,-100}},color = {95,95,95}));
          connect(hubAngles_3.hubFlange,hub_3) annotation(Line(points = {{30,-70},{30,-100}},color = {95,95,95}));
          connect(hubAngles_4.hubFlange,hub_4) annotation(Line(points = {{70,-70},{70,-100}},color = {95,95,95}));
          connect(hubAngles_1.steer,wheelOutputs[1].phi_str) annotation(Line(points = {{-49,-74},{-40,-74},{-40,-24},{70,-24}},color = {0,0,127}));
          connect(hubAngles_2.steer,wheelOutputs[2].phi_str) annotation(Line(points = {{-9,-74},{-4,-74},{-4,-24},{70,-24}},color = {0,0,127}));
          connect(hubAngles_3.steer,wheelOutputs[3].phi_str) annotation(Line(points = {{9,-74},{4,-74},{4,-24},{70,-24}},color = {0,0,127}));
          connect(hubAngles_4.steer,wheelOutputs[4].phi_str) annotation(Line(points = {{49,-74},{38,-74},{38,-24},{70,-24}},color = {0,0,127}));
          connect(v_pack.systemBus,controlBus) annotation(Line(points = {{-12,8},{-74,8},{-74,-60},{-100,-60}},color = {240,170,40},pattern = LinePattern.Dot));
          connect(v_pack.y_r,vehicleOutputs.v_bat) annotation(Line(points = {{-3,8},{20,8},{20,0},{100,0}},color = {0,0,127}));

          connect(wheelOutputs[1].sr,signalBus.VDL_whl_slp_x[1]) annotation(Line(points = {{70,-24},{50,-24},{50,-38},{-74,-38},{-74,60},{-100,60}},color = {0,0,127}));
          connect(wheelOutputs[2].sr,signalBus.VDL_whl_slp_x[2]) annotation(Line(points = {{70,-24},{50,-24},{50,-38},{-74,-38},{-74,60},{-100,60}},color = {0,0,127}));
          connect(wheelOutputs[3].sr,signalBus.VDL_whl_slp_x[3]) annotation(Line(points = {{70,-24},{50,-24},{50,-38},{-74,-38},{-74,60},{-100,60}},color = {0,0,127}));
          connect(wheelOutputs[4].sr,signalBus.VDL_whl_slp_x[4]) annotation(Line(points = {{70,-24},{50,-24},{50,-38},{-74,-38},{-74,60},{-100,60}},color = {0,0,127}));

          connect(wheelOutputs[1].omg,signalBus.VDL_whl_spd[1]) annotation(Line(points = {{70,-24},{50,-24},{50,-38},{-74,-38},{-74,60},{-100,60}},color = {0,0,127}));
          connect(wheelOutputs[2].omg,signalBus.VDL_whl_spd[2]) annotation(Line(points = {{70,-24},{50,-24},{50,-38},{-74,-38},{-74,60},{-100,60}},color = {0,0,127}));
          connect(wheelOutputs[3].omg,signalBus.VDL_whl_spd[3]) annotation(Line(points = {{70,-24},{50,-24},{50,-38},{-74,-38},{-74,60},{-100,60}},color = {0,0,127}));
          connect(wheelOutputs[4].omg,signalBus.VDL_whl_spd[4]) annotation(Line(points = {{70,-24},{50,-24},{50,-38},{-74,-38},{-74,60},{-100,60}},color = {0,0,127}));
          connect(driverOutputs.rV_x,vehicleOutputs.v_t_x) annotation(Line(points = {{0,110},{0,55},{100,55},{100,0}},color = {0,0,127}));
          connect(driverOutputs.rV_y,vehicleOutputs.v_t_y) annotation(Line(points = {{0,110},{0,55},{100,55},{100,0}},color = {0,0,127}));
          connect(driverOutputs.sG_x,vehicleOutputs.s_t_x) annotation(Line(points = {{0,110},{0,55},{100,55},{100,0}},color = {0,0,127}));
          connect(driverOutputs.sG_y,vehicleOutputs.s_t_y) annotation(Line(points = {{0,110},{0,55},{100,55},{100,0}},color = {0,0,127}));
          annotation(Icon(coordinateSystem(preserveAspectRatio = false,extent = {{-100.0,-100.0},{100.0,100.0}}),graphics={  Text(lineColor={0,0,255},extent={{-150,150},{150,110}},textString="%name")}));
      end SensorPackage;

      model DriverCommands
          .VehicleDynamics.Drivers.Tracking.Blocks.SinglePointLateralTracker lateralTracker annotation(Placement(transformation(extent = {{-12.0,28.0},{8.0,48.0}},origin={20,0},     rotation = 0.0)));
          .VehicleDynamics.Drivers.Tracking.Blocks.SinglePointLongitudinalTracker longitudinalTracker annotation(Placement(transformation(extent = {{-20.0,-40.0},{0.0,-20.0}},                   rotation = 0.0)));
          .VehicleDynamics.Drivers.Planning.DesiredVelocity planning(n_info = 1) annotation(Placement(transformation(extent = {{-54.0,0.0},{-34.0,20.0}},origin = {0.0,0.0},rotation = 0.0)));
          .Modelica.Blocks.Sources.Constant const(k = 0) annotation(Placement(transformation(extent = {{-60,-80},{-40,-60}},origin={4,0},  rotation = 0)));
          .CDC2023.Vehicles.Interfaces.DriverOutputs driverOutputs annotation(Placement(transformation(extent = {{100.0,-10.0},{120.0,10.0}},origin = {0.0,0.0},rotation = 0.0)));
          .VehicleDynamics.Drivers.Perception.Ideal perception annotation(Placement(transformation(extent = {{-92.0,0.0},{-72.0,20.0}},origin = {0.0,0.0},rotation = 0.0)));
          .VehicleDynamics.Vehicles.Interfaces.SignalBus signalBus annotation(Placement(transformation(extent = {{-10.0,-10.0},{10.0,10.0}},origin = {-100.0,0.0},rotation = -90.0)));
          .Modelica.Mechanics.MultiBody.Interfaces.Frame_a frame_a annotation(Placement(transformation(extent = {{-16.0,-16.0},{16.0,16.0}},origin = {0.0,-100.0},rotation = 90.0)));
          .Modelica.Blocks.Interfaces.RealInput desired_velocity annotation(Placement(transformation(extent = {{-11.632925860839833,-11.632925860839833},{11.632925860839833,11.632925860839833}},origin = {-40.0,112.0},rotation = -90.0)));
          .Modelica.Blocks.Math.Gain gain(k = 1 / 500) annotation(Placement(transformation(extent = {{19.2747110706908,-30.7252889293092},{28.7252889293092,-21.2747110706908}},origin = {0.0,0.0},rotation = 0.0)));
          .VehicleDynamics.Drivers.Planning.FromGround lateralPlanning annotation(Placement(transformation(extent = {{-48.0,58.0},{-28.0,78.0}},origin = {0.0,0.0},rotation = 0.0)));
          .VehicleDynamics.Drivers.Planning.DesiredVelocity planning2(n_info = 1) annotation(Placement(transformation(extent = {{-54.0,-30.0},{-34.0,-10.0}},origin = {0.0,0.0},rotation = 0.0)));
          .Modelica.Blocks.Interfaces.RealInput desired_velocity_preview annotation(Placement(transformation(extent = {{-11.632925860839833,-11.632925860839833},{11.632925860839833,11.632925860839833}},origin = {-10.0,112.0},rotation = -90.0)));
      protected
        VehicleDynamics.Drivers.Perception.Interfaces.PerceptsOut perceptsOut
          annotation (Placement(transformation(extent={{60,-56},{80,-36}})));
        VehicleDynamics.Drivers.Planning.Interfaces.PathPointOut pathPointOut[3]
          annotation (Placement(transformation(extent={{14.0,10.0},{34.0,30.0}},rotation = 0.0,origin = {0.0,0.0})));
      equation
          connect(lateralTracker.str_cmd,driverOutputs.str_cmd) annotation(Line(points={{29,38},
                {54,38},{54,0.05},{110.05,0.05}},                                                                            color = {0,0,127}));
          connect(longitudinalTracker.acc_cmd,driverOutputs.acc_cmd) annotation(Line(points={{1,-34},
                {55.5,-34},{55.5,0.05},{110.05,0.05}},                                                                              color = {0,0,127}));

          connect(pathPointOut[2].vV_P_x,driverOutputs.vV_P_x[1]) annotation(Line(points={{24,20},{36,20},{36,0.05},{110.05,0.05}},                                                                      color = {255,0,0}));
          connect(pathPointOut[3].vV_P_x,driverOutputs.vV_P_x[2]) annotation(Line(points={{24,20},{36,20},{36,0.05},{110.05,0.05}},                                                                      color = {255,0,0}));

          connect(pathPointOut[1].rV_x,driverOutputs.rV_x) annotation(Line(points={{24,20},{36,20},{36,0.05},{110.05,0.05}},                                                               color = {255,0,0}));
          connect(pathPointOut[1].rV_y,driverOutputs.rV_y) annotation(Line(points={{24,20},{36,20},{36,0.05},{110.05,0.05}},                                                               color = {255,0,0}));

          connect(lateralTracker.previewDistance[1],driverOutputs.previewDistance_y) annotation(Line(points={{14,49},
                {14,58},{96,58},{96,0.05},{110.05,0.05}},                                                                                              color = {0,0,127}));
          connect(lateralTracker.previewTime[1],driverOutputs.previewTime_y) annotation(Line(points={{22,49},
                {22,54},{94,54},{94,0.05},{110.05,0.05}},                                                                                    color = {0,0,127}));

          connect(longitudinalTracker.previewTime,driverOutputs.previewTime) annotation(Line(points={{-6,-19},
                {-6,-4},{46,-4},{46,0.05},{110.05,0.05}},                                                                                       color = {0,0,127}));
          connect(longitudinalTracker.previewDistance,driverOutputs.previewDistance) annotation(Line(points={{-14,-19},
                {-14,0},{56,0},{56,0.05},{110.05,0.05}},                                                                                                  color = {0,0,127}));

          connect(const.y,longitudinalTracker.acc_trk) annotation(Line(points={{-35,-70},
                {-14,-70},{-14,-41}},                                                                           color = {0,0,127}));
          connect(const.y,longitudinalTracker.brk_trk) annotation(Line(points={{-35,-70},
                {-6,-70},{-6,-41.1}},                                                                           color = {0,0,127}));
          connect(perception.percepts,planning.percepts) annotation(Line(points = {{-70,10},{-56,10}},color = {0,128,255},pattern = LinePattern.Dash));
          connect(perception.perceptionFrame,frame_a) annotation(Line(points = {{-82,0},{-82,-90},{0,-90},{0,-100}},color = {95,95,95}));
          connect(perception.dashboardInformation,signalBus.dashboard) annotation(Line(points = {{-82,20},{-82,26},{-100,26},{-100,0}},color = {0,0,0}));

        connect(lateralTracker.percepts, perception.percepts) annotation (Line(
            points={{6,34},{-64,34},{-64,10},{-70,10}},
            color={0,128,255},
            pattern=LinePattern.Dash,
            thickness=0.5));
        connect(longitudinalTracker.percepts, perception.percepts) annotation (Line(
            points={{-22,-34},{-70,-34},{-70,10}},
            color={0,128,255},
            pattern=LinePattern.Dash,
            thickness=0.5));
        connect(perceptsOut, perception.percepts) annotation (Line(
            points={{70,-46},{-70,-46},{-70,10}},
            color={0,128,255},
            pattern=LinePattern.Dash,
            thickness=0.5));
        connect(perceptsOut.v_veh, driverOutputs.v_veh_percepts) annotation (Line(
            points={{70.05,-45.95},{70.05,0.05},{110.05,0.05}},
            color={0,128,255},
            pattern=LinePattern.Dash,
            thickness=0.5));
        connect(perceptsOut.vV_x, driverOutputs.vV_x_percepts) annotation (Line(
            points={{70.05,-45.95},{70.05,0.05},{110.05,0.05}},
            color={0,128,255},
            pattern=LinePattern.Dash,
            thickness=0.5));
        connect(perceptsOut.vV_y, driverOutputs.vV_y_percepts) annotation (Line(
            points={{70.05,-45.95},{70.05,0.05},{110.05,0.05}},
            color={0,128,255},
            pattern=LinePattern.Dash,
            thickness=0.5));
        connect(driverOutputs.vV_P_x_percepts, perceptsOut.vV_P_x) annotation (Line(
              points={{110.05,0.05},{70.05,0.05},{70.05,-45.95}}, color={0,0,127}));
          connect(planning.pathVelocity,desired_velocity) annotation(Line(points = {{-55,14},{-66,14},{-66,92},{-40,92},{-40,112}},color = {0,0,127}));
          connect(gain.y,driverOutputs.brk_cmd) annotation(Line(points = {{29.197817822240122,-26},{56,-26},{56,0},{110,0}},color = {0,0,127}));
          connect(gain.u,longitudinalTracker.brk_cmd) annotation(Line(points = {{18.329653284828957,-26},{1,-26}},color = {0,0,127}));
          connect(lateralTracker.previewDistance,lateralPlanning.previewDistance) annotation(Line(points = {{14,49},{14,85},{-34,85},{-34,79}},color = {0,0,127}));
          connect(lateralTracker.previewTime,lateralPlanning.previewTime) annotation(Line(points = {{22,49},{22,88},{-42,88},{-42,79}},color = {0,0,127}));
          connect(lateralPlanning.pathPoint,lateralTracker.pathPoint) annotation(Line(points = {{-26,68},{-18,68},{-18,42},{6,42}},color = {255,0,0}));
          connect(longitudinalTracker.previewDistance[1],planning.previewDistance[1]) annotation(Line(points = {{-14,-19},{-14,27},{-40,27},{-40,21}},color = {0,0,127}));
          connect(longitudinalTracker.previewTime[1],planning.previewTime[1]) annotation(Line(points = {{-6,-19},{-6,27},{-48,27},{-48,21}},color = {0,0,127}));
          connect(longitudinalTracker.previewDistance[2],planning2.previewDistance[1]) annotation(Line(points = {{-14,-19},{-14,-4},{-40,-4},{-40,-9}},color = {0,0,127}));
          connect(longitudinalTracker.previewTime[2],planning2.previewTime[1]) annotation(Line(points = {{-6,-19},{-6,-3},{-48,-3},{-48,-9}},color = {0,0,127}));
          connect(desired_velocity_preview,planning2.pathVelocity) annotation(Line(points = {{-10,112},{-10,90},{-60,90},{-60,-16},{-55,-16}},color = {0,0,127}));
          connect(lateralPlanning.percepts,perception.percepts) annotation(Line(points = {{-50,68},{-64,68},{-64,10},{-70,10}},color = {0,128,255},pattern = LinePattern.Dash));
          connect(planning2.percepts,perception.percepts) annotation(Line(points = {{-56,-20},{-64,-20},{-64,10},{-70,10}},color = {0,128,255},pattern = LinePattern.Dash));
          connect(longitudinalTracker.pathPoint[1],planning.pathPoint[1]) annotation(Line(points = {{-22,-26},{-28,-26},{-28,10},{-32,10}},color = {255,0,0}));
          connect(longitudinalTracker.pathPoint[2],planning2.pathPoint[1]) annotation(Line(points = {{-22,-26},{-28,-26},{-28,-20},{-32,-20}},color = {255,0,0}));
          connect(lateralPlanning.pathPoint[1],pathPointOut[1]) annotation(Line(points = {{-26,68},{2,68},{2,20},{24,20}},color = {255,0,0}));
          connect(planning.pathPoint[1],pathPointOut[2]) annotation(Line(points = {{-32,10},{0,10},{0,20},{24,20}},color = {255,0,0}));
          connect(planning2.pathPoint[1],pathPointOut[3]) annotation(Line(points = {{-32,-20},{-28,-20},{-28,10},{0,10},{0,20},{24,20}},color = {255,0,0}));
          connect(pathPointOut[1].sG_x,driverOutputs.sG_x) annotation(Line(points = {{24,20},{36,20},{36,0},{110,0}},color = {255,0,0}));
          connect(pathPointOut[1].sG_y,driverOutputs.sG_y) annotation(Line(points = {{24,20},{36,20},{36,0},{110,0}},color = {255,0,0}));
          annotation(Icon(coordinateSystem(preserveAspectRatio = false,extent = {{-100.0,-100.0},{100.0,100.0}}),graphics={  Rectangle(lineColor={0,0,0},fillColor={230,230,230},fillPattern=FillPattern.Solid,extent={{-100.0,-100.0},{100.0,100.0}}),Text(lineColor={0,0,255},extent={{-150,150},{150,110}},textString="%name")}));
      end DriverCommands;

      model Evaluation
          .CDC2023.Vehicles.Interfaces.VehicleOutputs vehicleOutputs annotation(Placement(transformation(extent = {{-120,50},{-100,70}},origin = {0,0},rotation = 0)));
          .CDC2023.Vehicles.Interfaces.DriverOutputs driverOutputs annotation(Placement(transformation(extent = {{-120.0,-50.0},{-100.0,-30.0}},origin = {0.0,0.0},rotation = 0.0)));
          .CDC2023.Vehicles.Interfaces.EvaluationOutputs evaluationOutputs annotation(Placement(transformation(extent = {{100,-10},{120,10}},origin = {0,0},rotation = 0)));
          .Modelica.Blocks.Math.Feedback velocityError annotation(Placement(transformation(extent = {{-30.0,-30.0},{-10.0,-10.0}},origin = {0.0,0.0},rotation = 0.0)));
          .Modelica.Blocks.Math.Product product[4] annotation(Placement(transformation(extent = {{-40.0,30.0},{-20.0,50.0}},origin = {0.0,0.0},rotation = 0.0)));
          .Modelica.Blocks.Routing.Multiplex4 currents annotation(Placement(transformation(extent = {{-80,50},{-60,70}},origin = {0,0},rotation = 0)));
          .Modelica.Blocks.Routing.Replicator voltage(nout = 4) annotation(Placement(transformation(extent = {{-80,20},{-60,40}},origin = {0,0},rotation = 0)));
          .Modelica.Blocks.Math.Sum sum(nin = 4) annotation(Placement(transformation(extent = {{-10.0,30.0},{10.0,50.0}},origin = {0.0,0.0},rotation = 0.0)));
          .Modelica.Blocks.Continuous.Integrator integrator annotation(Placement(transformation(extent = {{60.0,30.0},{80.0,50.0}},origin = {0.0,0.0},rotation = 0.0)));
          .Modelica.Blocks.Continuous.Integrator integrator2 annotation(Placement(transformation(extent = {{30.0,-30.0},{50.0,-10.0}},origin = {0.0,0.0},rotation = 0.0)));
          .Modelica.Blocks.Math.Product square2 annotation(Placement(transformation(extent = {{0.0,-30.0},{20.0,-10.0}},origin = {0.0,0.0},rotation = 0.0)));
          .Modelica.Mechanics.MultiBody.Interfaces.Frame_a frame_a annotation(Placement(transformation(extent = {{-16.0,-16.0},{16.0,16.0}},origin = {0.0,-100.0},rotation = 90.0)));
          .VehicleDynamics.Grounds.Sensors.PathInfo pathInfo annotation(Placement(transformation(extent = {{-40.0,-70.0},{-20.0,-50.0}},origin = {0.0,0.0},rotation = 0.0)));
          .Modelica.Blocks.Continuous.Integrator integrator3 annotation(Placement(transformation(extent = {{30.0,-70.0},{50.0,-50.0}},origin = {0.0,0.0},rotation = 0.0)));
          .Modelica.Blocks.Math.Product square3 annotation(Placement(transformation(extent = {{-6.0,-70.0},{14.0,-50.0}},origin = {0.0,0.0},rotation = 0.0)));
      equation
          connect(velocityError.u1,driverOutputs.vV_P_x[1]) annotation(Line(points = {{-28,-20},{-69,-20},{-69,-40},{-110,-40}},color = {0,0,127}));
          connect(velocityError.u2,driverOutputs.v_veh_percepts) annotation(Line(points = {{-20,-28},{-20,-40},{-110,-40}},color = {0,0,127}));
          connect(currents.u1[1],vehicleOutputs.FL_wheel.i) annotation(Line(points = {{-82,69},{-82,75},{-110,75},{-110,60}},color = {0,0,127}));
          connect(currents.u2[1],vehicleOutputs.FR_wheel.i) annotation(Line(points = {{-82,63},{-96,63},{-96,60},{-110,60}},color = {0,0,127}));
          connect(currents.u3[1],vehicleOutputs.RL_wheel.i) annotation(Line(points = {{-82,57},{-96,57},{-96,60},{-110,60}},color = {0,0,127}));
          connect(currents.u4[1],vehicleOutputs.RR_wheel.i) annotation(Line(points = {{-82,51},{-82,45},{-110,45},{-110,60}},color = {0,0,127}));
          connect(currents.y,product.u1) annotation(Line(points = {{-59,60},{-50.5,60},{-50.5,46},{-42,46}},color = {0,0,127}));
          connect(voltage.u,vehicleOutputs.v_bat) annotation(Line(points = {{-82,30},{-96,30},{-96,60},{-110,60}},color = {0,0,127}));
          connect(voltage.y,product.u2) annotation(Line(points = {{-59,30},{-50.5,30},{-50.5,34},{-42,34}},color = {0,0,127}));
          connect(sum.u,product.y) annotation(Line(points = {{-12,40},{-19,40}},color = {0,0,127}));
          connect(integrator.y,evaluationOutputs.J_e) annotation(Line(points = {{81,40},{96,40},{96,0},{110,0}},color = {0,0,127}));
          connect(square2.u1,velocityError.y) annotation(Line(points = {{-2,-14},{-6.5,-14},{-6.5,-20},{-11,-20}},color = {0,0,127}));
          connect(square2.u2,velocityError.y) annotation(Line(points = {{-2,-26},{-6.5,-26},{-6.5,-20},{-11,-20}},color = {0,0,127}));
          connect(square2.y,integrator2.u) annotation(Line(points = {{21,-20},{28,-20}},color = {0,0,127}));
          connect(integrator2.y,evaluationOutputs.J_v_1) annotation(Line(points = {{51,-20},{96,-20},{96,0},{110,0}},color = {0,0,127}));
          connect(pathInfo.mountFrame,frame_a) annotation(Line(points = {{-40,-60},{-80,-60},{-80,-86},{0,-86},{0,-100}},color = {95,95,95}));
          connect(integrator3.y,evaluationOutputs.J_v_2) annotation(Line(points = {{51,-60},{96,-60},{96,0},{110,0}},color = {0,0,127}));
          connect(pathInfo.r_P[2],square3.u1) annotation(Line(points = {{-19,-60},{-13.5,-60},{-13.5,-54},{-8,-54}},color = {0,0,127}));
          connect(pathInfo.r_P[2],square3.u2) annotation(Line(points = {{-19,-60},{-13.5,-60},{-13.5,-66},{-8,-66}},color = {0,0,127}));
          connect(square3.y,integrator3.u) annotation(Line(points = {{15,-60},{28,-60}},color = {0,0,127}));
          connect(sum.y,integrator.u) annotation(Line(points = {{11,40},{58,40}},color = {0,0,127}));
          annotation(Icon(coordinateSystem(preserveAspectRatio = false,extent = {{-100.0,-100.0},{100.0,100.0}}),graphics={  Rectangle(lineColor={0,0,0},fillColor={230,230,230},fillPattern=FillPattern.Solid,extent={{-100.0,-100.0},{100.0,100.0}}),Text(lineColor={0,0,255},extent={{-150,150},{150,110}},textString="%name")}));
      end Evaluation;
    end Sensors;

    package Exports
        extends Modelon.Icons.ExamplesPackage;
      model Sedan
          extends .VehicleDynamics.Vehicles.Experiments.Templates.Minimal(redeclare replaceable .VehicleDynamics.Atmospheres.Constant atmosphere,redeclare replaceable .VehicleDynamics.Grounds.Flat ground,redeclare .CDC2023.Vehicles.SedanInWheelMotors vehicle(v_start = 16.66666666666666666667),world(enableAnimation = false));
          .VehicleDynamics.Vehicles.Interfaces.SignalBus signalBus annotation(Placement(transformation(extent = {{-90.0,-30.0},{-70.0,-10.0}},origin = {0.0,0.0},rotation = 0.0)));
          .VehicleDynamics.Vehicles.Chassis.Experiments.Robots.SteerServo steerRobot "Robot for steering" annotation(Placement(transformation(extent = {{-60.0,10.0},{-40.0,30.0}},rotation = 0.0,origin = {0.0,0.0})));
          .Modelica.Blocks.Sources.BooleanConstant release(k = false) annotation(Placement(transformation(extent = {{-80.0,80.0},{-60.0,100.0}},rotation = 0.0,origin = {0.0,0.0})));
        .CDC2023.Vehicles.Sensors.DriverCommands driverCommands
          annotation (Placement(transformation(extent={{10.0,10.0},{30.0,30.0}},rotation = 0.0,origin = {0.0,0.0})));
          .Modelica.Blocks.Interfaces.RealInput phi_str annotation(Placement(transformation(extent = {{-140.0,50.0},{-100.0,90.0}},origin = {0.0,0.0},rotation = 0.0)));
          .Modelica.Blocks.Interfaces.RealInput tau_FL annotation(Placement(transformation(extent = {{-140.0,16.0},{-100.0,56.0}},origin = {0.0,0.0},rotation = 0.0)));
          .Modelica.Blocks.Interfaces.RealInput tau_FR annotation(Placement(transformation(extent = {{-140.0,-20.0},{-100.0,20.0}},origin = {0.0,0.0},rotation = 0.0)));
          .Modelica.Blocks.Interfaces.RealInput tau_RL annotation(Placement(transformation(extent = {{-140.0,-56.0},{-100.0,-16.0}},origin = {0.0,0.0},rotation = 0.0)));
          .Modelica.Blocks.Interfaces.RealInput tau_RR annotation(Placement(transformation(extent = {{-140.0,-90.0},{-100.0,-50.0}},origin = {0.0,0.0},rotation = 0.0)));
          .CDC2023.Vehicles.Interfaces.VehicleOutputs vehicleOutputs annotation(Placement(transformation(extent = {{100,-30},{120,-10}},origin = {0,0},rotation = 0)));
          .CDC2023.Vehicles.Interfaces.DriverOutputs driverOutputs annotation(Placement(transformation(extent = {{100,30},{120,50}},origin = {0,0},rotation = 0)));
          replaceable .Modelica.Blocks.Sources.Constant desiredVelocity(k = vehicle.v_start) constrainedby .Modelica.Blocks.Interfaces.SO annotation(Placement(transformation(extent = {{-20.0,30.0},{0.0,50.0}},origin = {0.0,0.0},rotation = 0.0)));
          replaceable .Modelica.Blocks.Sources.Constant desiredVelocityPreview(k = vehicle.v_start) constrainedby .Modelica.Blocks.Interfaces.SO annotation(Placement(transformation(extent = {{-20.0,60.0},{0.0,80.0}},origin = {0.0,0.0},rotation = 0.0)));
          .CDC2023.Vehicles.Sensors.Evaluation evaluation annotation(Placement(transformation(extent = {{50.0,-20.0},{70.0,0.0}},origin = {0.0,0.0},rotation = 0.0)));
          .CDC2023.Vehicles.Interfaces.EvaluationOutputs evaluationOutputs annotation(Placement(transformation(extent = {{100,0},{120,20}},origin = {0,0},rotation = 0)));
          .CDC2023.Vehicles.Sensors.SensorPackage sensorPackage annotation(Placement(transformation(extent = {{66.0,-58.0},{86.0,-38.0}},origin = {0.0,0.0},rotation = 0.0)));
      equation
          connect(vehicle.signalBus,signalBus) annotation(Line(points = {{-60,-70},{-80,-70},{-80,-20}},color = {255,204,51}));
          connect(release.y,steerRobot.release) annotation(Line(points = {{-59,90},{-50,90},{-50,31}},color = {255,0,255}));
          connect(vehicle.steeringWheelFlange,steerRobot.steeringWheelFlange) annotation(Line(points = {{-35,-30},{-35,20},{-40,20}},color = {95,95,95}));
        connect(driverCommands.frame_a, vehicle.vehicleFrame) annotation (Line(
            points={{20,10},{20,0},{-68,0},{-68,-55},{-60,-55}},
            color={95,95,95},
            thickness=0.5));
          connect(steerRobot.phi_ref,phi_str) annotation(Line(points = {{-61,20},{-68,20},{-68,70},{-120,70}},color = {0,0,127}));
          connect(tau_FL,signalBus.VDL_trm_mtr_trq_cmd[1]) annotation(Line(points = {{-120,36},{-80,36},{-80,-20}},color = {0,0,127}));
          connect(tau_FR,signalBus.VDL_trm_mtr_trq_cmd[2]) annotation(Line(points = {{-120,0},{-80,0},{-80,-20}},color = {0,0,127}));
          connect(tau_RL,signalBus.VDL_trm_mtr_trq_cmd[3]) annotation(Line(points = {{-120,-36},{-80,-36},{-80,-20}},color = {0,0,127}));
          connect(tau_RR,signalBus.VDL_trm_mtr_trq_cmd[4]) annotation(Line(points = {{-120,-70},{-80,-70},{-80,-20}},color = {0,0,127}));
          connect(driverCommands.signalBus,vehicle.signalBus) annotation(Line(points = {{10,20},{-70,20},{-70,-70},{-60,-70}},color = {255,204,51}));
          connect(driverCommands.driverOutputs,driverOutputs) annotation(Line(points = {{31,20},{76,20},{76,40},{110,40}},color = {0,0,127}));
          connect(desiredVelocity.y,driverCommands.desired_velocity) annotation(Line(points = {{1,40},{16,40},{16,31.200000000000003}},color = {0,0,127}));
          connect(desiredVelocityPreview.y,driverCommands.desired_velocity_preview) annotation(Line(points = {{1,70},{19,70},{19,31.200000000000003}},color = {0,0,127}));
          connect(evaluation.frame_a,vehicle.vehicleFrame) annotation(Line(points = {{60,-20},{60,-24},{-68,-24},{-68,-55},{-60,-55}},color = {95,95,95}));
          connect(evaluation.driverOutputs,driverCommands.driverOutputs) annotation(Line(points = {{49,-14},{40,-14},{40,20},{31,20}},color = {0,0,127}));
          connect(evaluation.evaluationOutputs,evaluationOutputs) annotation(Line(points = {{71,-10},{90.5,-10},{90.5,10},{110,10}},color = {0,0,127}));
          connect(sensorPackage.vehicleOutputs,vehicleOutputs) annotation(Line(points = {{86,-48},{95,-48},{95,-20},{110,-20}},color = {0,0,127}));
          connect(sensorPackage.signalBus,vehicle.signalBus) annotation(Line(points = {{66,-42},{48,-42},{48,-90},{-66,-90},{-66,-70},{-60,-70}},color = {255,204,51}));
          connect(sensorPackage.controlBus,vehicle.controlBus) annotation(Line(points = {{66,-54},{54,-54},{54,-24},{0,-24},{0,-30}},color = {240,170,40},pattern = LinePattern.Dot));
          connect(sensorPackage.frame_a,vehicle.vehicleFrame) annotation(Line(points = {{66,-48},{46,-48},{46,-24},{-66,-24},{-66,-55},{-60,-55}},color = {95,95,95}));
          connect(sensorPackage.hub_1,vehicle.hub_1) annotation(Line(points = {{69,-58},{69,-64},{52,-64},{52,-86},{-45,-86},{-45,-80}},color = {95,95,95}));
          connect(sensorPackage.hub_2,vehicle.hub_2) annotation(Line(points = {{73,-58},{73,-64},{11,-64},{11,-24},{-45,-24},{-45,-30}},color = {95,95,95}));
          connect(sensorPackage.hub_3,vehicle.hub_3) annotation(Line(points = {{79,-58},{79,-64},{52,-64},{52,-86},{25,-86},{25,-80}},color = {95,95,95}));
          connect(sensorPackage.hub_4,vehicle.hub_4) annotation(Line(points = {{83,-58},{83,-64},{52,-64},{52,-24},{25,-24},{25,-30}},color = {95,95,95}));
          connect(sensorPackage.driverOutputs,driverCommands.driverOutputs) annotation(Line(points = {{76,-37},{76,20},{31,20}},color = {0,0,127}));
          connect(sensorPackage.vehicleOutputs,evaluation.vehicleOutputs) annotation(Line(points = {{86,-48},{92,-48},{92,-26},{44,-26},{44,-4},{49,-4}},color = {0,0,127}));
      end Sedan;

      model DoubleLaneChange
          extends .CDC2023.Vehicles.Exports.Sedan(redeclare replaceable .CDC2023.Grounds.OpenCRGUneven ground(file_name = Modelica.Utilities.Files.loadResource("modelica://VehicleDynamics/Resources/Data/Roads/crg-txt/flat_250.crg"),override_offset = false,path_file = Modelica.Utilities.Files.loadResource("modelica://VehicleDynamics/Resources/Data/Roads/crg-pth/doubleLaneChange.mat"),path_velocity = 16.66666666666666666667,visualize_cones = true,visualize_path = true),vehicle(r_y_start = -2));
      end DoubleLaneChange;

      model Acceleration
          extends .CDC2023.Vehicles.Exports.Sedan(redeclare replaceable .Modelica.Blocks.Sources.Trapezoid desiredVelocity(startTime = 5,nperiod = 1,rising = 5,falling = 5,width = 5,amplitude = 15,period = 20),vehicle(v_start = 0),redeclare replaceable .Modelica.Blocks.Sources.Trapezoid desiredVelocityPreview(amplitude = desiredVelocity.amplitude,rising = desiredVelocity.rising,width = desiredVelocity.width,falling = desiredVelocity.falling,period = desiredVelocity.period,nperiod = desiredVelocity.nperiod,offset = desiredVelocity.offset,startTime = desiredVelocity.startTime - 1),redeclare replaceable .CDC2023.Grounds.OpenCRGUneven ground(file_name = Modelica.Utilities.Files.loadResource("modelica://VehicleDynamics/Resources/Data/Roads/crg-txt/flat_250.crg")));
      end Acceleration;

            model DoubleLaneChangeMap
              extends .CDC2023.Vehicles.Exports.Sedan(redeclare replaceable .CDC2023.Grounds.WavyUneven ground(file_name = .Modelica.Utilities.Files.loadResource("modelica://VehicleDynamics/Resources/Data/Roads/Examples_DoubleLaneChange.mat"), xlims = {-5, 200}, ylims = {-4, 4}), vehicle(r_y_start = -2));
            end DoubleLaneChangeMap;

            model AccelerationMap
              extends .CDC2023.Vehicles.Exports.Sedan(redeclare replaceable .Modelica.Blocks.Sources.Trapezoid desiredVelocity(startTime = 5, nperiod = 1, rising = 5, falling = 5, width = 5, amplitude = 15, period = 20), vehicle(v_start = 0), redeclare replaceable .Modelica.Blocks.Sources.Trapezoid desiredVelocityPreview(amplitude = desiredVelocity.amplitude, rising = desiredVelocity.rising, width = desiredVelocity.width, falling = desiredVelocity.falling, period = desiredVelocity.period, nperiod = desiredVelocity.nperiod, offset = desiredVelocity.offset, startTime = desiredVelocity.startTime - 1), redeclare replaceable .CDC2023.Grounds.WavyUneven ground(file_name = .Modelica.Utilities.Files.loadResource("modelica://VehicleDynamics/Resources/Data/Roads/Examples_Straight.mat"), ylims = {-3, 3}, xlims = {-5, 175}));
            end AccelerationMap;

      package Tests
          extends Modelon.Icons.ExperimentsPackage;
        model DoubleLaneChange
            extends Modelon.Icons.Experiment;
            .CDC2023.Vehicles.Exports.DoubleLaneChange doubleLaneChange(world(enableAnimation = true)) annotation(Placement(transformation(extent = {{-8,-8},{12,12}},origin = {0,0},rotation = 0)));
            .Modelica.Blocks.Math.Add add(k1 = 100,k2 = -1500) annotation(Placement(transformation(extent = {{-86,-16},{-66,4}},origin = {0,0},rotation = 0)));
        equation
            connect(doubleLaneChange.driverOutputs.str_cmd,doubleLaneChange.phi_str) annotation(Line(points = {{13,6},{64,6},{64,-66},{-54,-66},{-54,9},{-10,9}},color = {0,0,127}));
            connect(add.y,doubleLaneChange.tau_FL) annotation(Line(points = {{-65,-6},{-37.5,-6},{-37.5,5.6000000000000005},{-10,5.6000000000000005}},color = {0,0,127}));
            connect(add.y,doubleLaneChange.tau_FR) annotation(Line(points = {{-65,-6},{-37.5,-6},{-37.5,2},{-10,2}},color = {0,0,127}));
            connect(add.y,doubleLaneChange.tau_RL) annotation(Line(points = {{-65,-6},{-37.5,-6},{-37.5,-1.6000000000000005},{-10,-1.6000000000000005}},color = {0,0,127}));
            connect(add.y,doubleLaneChange.tau_RR) annotation(Line(points = {{-65,-6},{-37.5,-6},{-37.5,-5},{-10,-5}},color = {0,0,127}));
            connect(doubleLaneChange.driverOutputs.brk_cmd,add.u2) annotation(Line(points = {{13,6},{19,6},{19,18},{-92,18},{-92,-12},{-88,-12}},color = {0,0,127}));
            connect(doubleLaneChange.driverOutputs.acc_cmd,add.u1) annotation(Line(points = {{13,6},{19,6},{19,18},{-92,18},{-92,0},{-88,0}},color = {0,0,127}));
        end DoubleLaneChange;

        model Acceleration
            extends Modelon.Icons.Experiment;
            .CDC2023.Vehicles.Exports.Acceleration acceleration(driverCommands(longitudinalTracker(K_acc = 1)),world(enableAnimation = true)) annotation(Placement(transformation(extent = {{-8,-8},{12,12}},origin = {0,0},rotation = 0)));
            .Modelica.Blocks.Math.Add add(k1 = 100,k2 = -1500) annotation(Placement(transformation(extent = {{-86,-16},{-66,4}},origin = {0,0},rotation = 0)));
        equation
            connect(acceleration.driverOutputs.str_cmd,acceleration.phi_str) annotation(Line(points = {{13,6},{64,6},{64,-66},{-54,-66},{-54,9},{-10,9}},color = {0,0,127}));
            connect(add.y,acceleration.tau_FL) annotation(Line(points = {{-65,-6},{-37.5,-6},{-37.5,5.6000000000000005},{-10,5.6000000000000005}},color = {0,0,127}));
            connect(add.y,acceleration.tau_FR) annotation(Line(points = {{-65,-6},{-37.5,-6},{-37.5,2},{-10,2}},color = {0,0,127}));
            connect(add.y,acceleration.tau_RL) annotation(Line(points = {{-65,-6},{-37.5,-6},{-37.5,-1.6000000000000005},{-10,-1.6000000000000005}},color = {0,0,127}));
            connect(add.y,acceleration.tau_RR) annotation(Line(points = {{-65,-6},{-37.5,-6},{-37.5,-5},{-10,-5}},color = {0,0,127}));
            connect(acceleration.driverOutputs.brk_cmd,add.u2) annotation(Line(points = {{13,6},{19,6},{19,18},{-92,18},{-92,-12},{-88,-12}},color = {0,0,127}));
            connect(acceleration.driverOutputs.acc_cmd,add.u1) annotation(Line(points = {{13,6},{19,6},{19,18},{-92,18},{-92,0},{-88,0}},color = {0,0,127}));
        end Acceleration;

              model DoubleLaneChange_MapRoad
                  extends .Modelon.Icons.Experiment;
                  .CDC2023.Vehicles.Exports.DoubleLaneChangeMap doubleLaneChange(world(enableAnimation = true)) annotation(Placement(transformation(extent = {{-8,-8},{12,12}},origin = {0,0},rotation = 0)));
                  .Modelica.Blocks.Math.Add add(k1 = 100,k2 = -1500) annotation(Placement(transformation(extent = {{-86,-16},{-66,4}},origin = {0,0},rotation = 0)));
              equation
                  connect(doubleLaneChange.driverOutputs.str_cmd,doubleLaneChange.phi_str) annotation(Line(points = {{13,6},{64,6},{64,-66},{-54,-66},{-54,9},{-10,9}},color = {0,0,127}));
                  connect(add.y,doubleLaneChange.tau_FL) annotation(Line(points = {{-65,-6},{-37.5,-6},{-37.5,5.6000000000000005},{-10,5.6000000000000005}},color = {0,0,127}));
                  connect(add.y,doubleLaneChange.tau_FR) annotation(Line(points = {{-65,-6},{-37.5,-6},{-37.5,2},{-10,2}},color = {0,0,127}));
                  connect(add.y,doubleLaneChange.tau_RL) annotation(Line(points = {{-65,-6},{-37.5,-6},{-37.5,-1.6000000000000005},{-10,-1.6000000000000005}},color = {0,0,127}));
                  connect(add.y,doubleLaneChange.tau_RR) annotation(Line(points = {{-65,-6},{-37.5,-6},{-37.5,-5},{-10,-5}},color = {0,0,127}));
                  connect(doubleLaneChange.driverOutputs.brk_cmd,add.u2) annotation(Line(points = {{13,6},{19,6},{19,18},{-92,18},{-92,-12},{-88,-12}},color = {0,0,127}));
                  connect(doubleLaneChange.driverOutputs.acc_cmd,add.u1) annotation(Line(points = {{13,6},{19,6},{19,18},{-92,18},{-92,0},{-88,0}},color = {0,0,127}));
              end DoubleLaneChange_MapRoad;

              model Acceleration_MapRoad
                  extends .Modelon.Icons.Experiment;
                  .CDC2023.Vehicles.Exports.AccelerationMap acceleration(driverCommands(longitudinalTracker(K_acc = 1)),world(enableAnimation = true)) annotation(Placement(transformation(extent = {{-8,-8},{12,12}},origin = {0,0},rotation = 0)));
                  .Modelica.Blocks.Math.Add add(k1 = 100,k2 = -1500) annotation(Placement(transformation(extent = {{-86,-16},{-66,4}},origin = {0,0},rotation = 0)));
              equation
                  connect(acceleration.driverOutputs.str_cmd,acceleration.phi_str) annotation(Line(points = {{13,6},{64,6},{64,-66},{-54,-66},{-54,9},{-10,9}},color = {0,0,127}));
                  connect(add.y,acceleration.tau_FL) annotation(Line(points = {{-65,-6},{-37.5,-6},{-37.5,5.6000000000000005},{-10,5.6000000000000005}},color = {0,0,127}));
                  connect(add.y,acceleration.tau_FR) annotation(Line(points = {{-65,-6},{-37.5,-6},{-37.5,2},{-10,2}},color = {0,0,127}));
                  connect(add.y,acceleration.tau_RL) annotation(Line(points = {{-65,-6},{-37.5,-6},{-37.5,-1.6000000000000005},{-10,-1.6000000000000005}},color = {0,0,127}));
                  connect(add.y,acceleration.tau_RR) annotation(Line(points = {{-65,-6},{-37.5,-6},{-37.5,-5},{-10,-5}},color = {0,0,127}));
                  connect(acceleration.driverOutputs.brk_cmd,add.u2) annotation(Line(points = {{13,6},{19,6},{19,18},{-92,18},{-92,-12},{-88,-12}},color = {0,0,127}));
                  connect(acceleration.driverOutputs.acc_cmd,add.u1) annotation(Line(points = {{13,6},{19,6},{19,18},{-92,18},{-92,0},{-88,0}},color = {0,0,127}));
              end Acceleration_MapRoad;
      end Tests;
    end Exports;
  end Vehicles;

  package Grounds
      extends VehicleDynamics.Utilities.Icons.GroundsPackage;
    model WavyUneven "Wavy uneven ground with parametric wavelength and height"
      extends .VehicleDynamics.Grounds.Uneveness.TabularUneven(unevenness_table = [0, transpose([ylims[1]:dsy:ylims[2]]); xlims[1]:dsx:xlims[2], height*sin(2*.Modelica.Constants.pi*[xlims[1]:dsx:xlims[2]]/longitudinal_wavelength)*transpose(sin(2*.Modelica.Constants.pi*[ylims[1]:dsy:ylims[2]]/lateral_wavelength))]);
        parameter Real height = 0.01 "Height of uneveness";
        parameter .Modelica.Units.SI.Length longitudinal_wavelength = 2.7 "Wavelength of uneveness in longitudinal direction";
        parameter .Modelica.Units.SI.Length lateral_wavelength = 1.5 "Wavelength of uneveness in lateral direction";
        parameter .Modelica.Units.SI.Length dsx = 0.5 "Resolution of uneveness";
        parameter .Modelica.Units.SI.Length dsy = 0.25 "Resolution of uneveness";
        parameter .Modelica.Units.SI.Position[2] xlims = {-5,250} "Extent in longitudinal direction";
        parameter .Modelica.Units.SI.Position[2] ylims = {-5,5} "Extent in lateral direction";

    end WavyUneven;

    function rS_G_z "Flat (no) surface unevenness"
      extends VehicleDynamics.Grounds.Interfaces.rS_G_z;
      input Real height = 0.01 "Height of uneveness";
      input .Modelica.Units.SI.Length longitudinal_wavelength = 2.7 "Wavelength of uneveness in longitudinal direction";
      input .Modelica.Units.SI.Length lateral_wavelength = 1.5 "Wavelength of uneveness in lateral direction";
    algorithm
      rS_G_z := height*sin(2*.Modelica.Constants.pi*rG_G_x/longitudinal_wavelength)*sin(2*.Modelica.Constants.pi*rG_G_y/lateral_wavelength);
    end rS_G_z;

    model OpenCRGUneven
      extends VehicleDynamics.Grounds.OpenCRG(
        redeclare final function rS_G_z = CDC2023.Grounds.rS_G_z (
          height=height,
          longitudinal_wavelength=longitudinal_wavelength,
          lateral_wavelength=lateral_wavelength));
      parameter Real height = 0.01 "Height of uneveness";
      parameter .Modelica.Units.SI.Length longitudinal_wavelength = 2.7 "Wavelength of uneveness in longitudinal direction";
      parameter .Modelica.Units.SI.Length lateral_wavelength = 1.5 "Wavelength of uneveness in lateral direction";
    end OpenCRGUneven;
  end Grounds;
    annotation(uses(Modelica(version = "4.0.0"),VehicleDynamics(version = "4.1"),Electrification(version = "1.8"),Modelon(version = "4.1")),
      version="0.6");
end CDC2023;
